#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <algorithm>
#include <vector>
#include <string>
#include <cmath>
#include "simulation.h"

//Port for the link with Fluka with the script run.sh
#if defined(FLUKA)
#define PORT 12345
#endif
using namespace std;


//===============================Constructors, destructor======================


Simulation::Simulation(int size_bunch, int nbre_elts, string genere)
    : sizeBunch(size_bunch), nbre_elts(nbre_elts)
{
    double ax(-2.316794521);
    double bx(103.4788975);
    double dx(1.231007397);
    double dpx(0.027602708);
    double by(20.8994661);
    double ay(0.537534425);
    double dy(0);
    double dpy(0);
    srand(time(0));

    Particle p1;

    for (int l(0); l < sizeBunch; ++l) {
        /* generate the particle distributions */
        p1.genpartdist(1, 1, genere, 1.00095, 1.2e-8, 1.2e-8, 0.00011, bx, ax, dx, dpx, by, ay, dy, dpy, 6);
        p1.afficheFirstCoordonnees();

        bunch.push_back(p1);
    }
}

Simulation::~Simulation()
{

    for (int j(0); j < nbre_elts; ++j) {
        delete lat.reseau[j];
    }
}


void Simulation::die(const char* msg)
{
    cerr << msg << endl;

#if defined(FLUKA)
    if (lat.conn) {
        flukaio_disconnect(lat.conn);
        lat.conn = NULL;
    }
#endif

    exit(1);
}


/*
 void Simulation::readParticle(const double& Apr, const double& Zpr, string inputfile)
 
 */
void Simulation::readParticle(const double& Apr, const double& Zpr, string inputfile)
{

    string particlefile;

    particlefile = inputfile + "initial.dat";

    cout << "READ THE BUNCH OF PARTICLES FROM THE FILE: " << particlefile <<endl;
    
    //if the name of the file is not 'initial.dat', uncomment the following two lines

    //cout << "Enter the name of the file containing the particles informations: " << endl;

    //cin >> particlefile;

    ifstream enterPart;

    enterPart.open(particlefile.c_str());

    if (enterPart.fail()) {
        cerr << "Warning: problem reading the file " << particlefile << "!" << endl;
    } else {

        double temp2=0.0;
         //size_t  pos=0;
	// string input;
	
     /*read one cubic number of the file*/
        while (enterPart >> temp2) {

            Particle p;

            p.coordonnees[0][0] = temp2;
            enterPart >> p.coordonnees[0][1];
            enterPart >> p.coordonnees[0][2];
            enterPart >> p.coordonnees[0][3];
            enterPart >> p.coordonnees[0][4];
            p.Ap0 = Apr;
            p.Zp0 = Zpr;
            p.coordonnees[0][5] = 0;
            p.dt = 0;
            p.dpoporiginal = p.coordonnees[0][4];


            bunch.push_back(p);
        }
    }
    
    enterPart.close();
}

//read information from collimator file

void Simulation::readInputFile(string inputfile, double& Apr, double& Zpr, double& mass, double& energyPerIon, double& emix, double& emiy, long int& npart, int& kbunch, double& sigdpp, int& taubeam, long int& nparti, string& partdistr, double& r1r2skin, long int& nrev1, int& nrev2, double& blowup2, int& blowupperiod, string& stoplineartracking, int& scaleorbit, double& wecolli, double& nsigi, string& opticsFile, double& thicknessmagneticfield, double& Bmax, double& deltaGap, int& beamflag, int& RunWithFluka, int& RunWithCrystal, double& freqrf, double& rfvoltage, double& rfharmonic, int& RunningFlag, int& idpart, int& idelt, int& outcoord, double& momentum, string& plotflag, string& inputpath, int& RFflag)
{

    ifstream entree;
    string crosssectionspath;
    entree.open(inputfile.c_str());

    if (entree.fail()) {
        cerr << "Warning: problem to read the collimator file!!" << endl;
    } else {

        //reading the parameter values in the top part of the collimator file

        string name, comment, date;
	string strtemp,ws;
        size_t  pos=0;
	string  delimiter=",";
	
	entree >> ws;
        getline(entree, comment);
        getline(entree, date);

        while (entree >> ws) {
 
             pos=ws.find(delimiter);
            name=ws.substr(0,pos);
            ws.erase(0,pos+delimiter.size());
	    
	    strtemp=ws;
	    
	   // Fix the bugs to read long particle numbers & set the RF flag, by Jianfeng Zhang @ LAL, 15/04/2014
            if (name == "MASSNUMBER" && !strtemp.empty()) {
                Apr = atof(strtemp.c_str());
            } else if (name == "CHARGESTATE" && !strtemp.empty()) {
                Zpr = atof(strtemp.c_str());
            } else if (name == "MASS" && !strtemp.empty()) {
                mass = atof(strtemp.c_str());
            } else if (name == "ENERGY" && !strtemp.empty()) {
                energyPerIon = atof(strtemp.c_str());
            } else if (name == "EX" && !strtemp.empty()) {
                emix = atof(strtemp.c_str());
            } else if (name == "EY" && !strtemp.empty()) {
                emiy = atof(strtemp.c_str());
            } else if (name == "NPART" && !strtemp.empty()) {
                npart = atol(strtemp.c_str());
            } else if (name == "KBUNCH" && !strtemp.empty()) {
                kbunch = atoi(strtemp.c_str());
            } else if (name == "SIGDPP" && !strtemp.empty()) {
                sigdpp = atof(strtemp.c_str());
            } else if (name == "TAUBEAM" && !strtemp.empty()) {
                taubeam = atoi(strtemp.c_str());
            } else if (name == "NPARTI" && !strtemp.empty()) {
                nparti = atol(strtemp.c_str());
            } else if (name == "PARTDISTR" && !strtemp.empty()) {
		partdistr = strtemp;
            } else if (name == "R1R2SKIN" && !strtemp.empty()) {
                if (partdistr == "r1r2" && !strtemp.empty()) {
                    r1r2skin = atof(strtemp.c_str());
                } else {
              //      getline(entree, strtemp);
                }
            } else if (name == "NREV1" && !strtemp.empty()){ 
                nrev1 = atol(strtemp.c_str());
            } else if (name == "NREV2" && !strtemp.empty()) {
                nrev2 = atoi(strtemp.c_str());
            } else if (name == "BLOWUP2" && !strtemp.empty()) {
                blowup2 = atof(strtemp.c_str());
            } else if (name == "BLOWUPPERIOD" && !strtemp.empty()) {
                blowupperiod = atoi(strtemp.c_str());
            } else if (name == "STOPLINEARTRACKING" && !strtemp.empty()) {
             stoplineartracking = strtemp;
            } else if (name == "SCALEORBIT" && !strtemp.empty()) {
                scaleorbit = atoi(strtemp.c_str());
            } else if (name == "WECOLLI" && !strtemp.empty()) {
                wecolli = atof(strtemp.c_str());
            } else if (name == "NSIGI" && !strtemp.empty()) {
                nsigi = atof(strtemp.c_str());
            } else if (name == "CROSSSECTIONSPATH" && !strtemp.empty()) {
              crosssectionspath = strtemp;
            } else if (name == "OPTICSPATH" && !strtemp.empty()) {
              opticsFile = strtemp;
            } else if (name == "INPUTPATH" && !strtemp.empty()) {
             inputpath = strtemp;
            } else if (name == "THICKNESSMAGNETICFIELD" && !strtemp.empty()) {
                thicknessmagneticfield = atof(strtemp.c_str());
            } else if (name == "BMAX" && !strtemp.empty()) {
                Bmax = atof(strtemp.c_str());
            } else if (name == "DELTAGAP" && !strtemp.empty()) {
                deltaGap = atof(strtemp.c_str());
            } else if (name == "BEAMFLAG" && !strtemp.empty()) {
                beamflag = atoi(strtemp.c_str());
            } else if (name == "RUNNINGFLAG" && !strtemp.empty()) {
                RunningFlag = atoi(strtemp.c_str());
            } else if (name == "IDPART" && !strtemp.empty()) {
                idpart = atoi(strtemp.c_str());
            } else if (name == "OUTCOORD" && !strtemp.empty()) {
                outcoord = atoi(strtemp.c_str());
            } else if (name == "IDELT" && !strtemp.empty()) {
                idelt = atoi(strtemp.c_str());
            } else if (name == "FREQRF" && !strtemp.empty()) {
                freqrf = atof(strtemp.c_str());
                RFflag = 1;
            } else if (name == "RFVOLTAGE" && !strtemp.empty()) {
                rfvoltage = atof(strtemp.c_str());
                RFflag = 1;
            } else if (name == "RFHARMONBER" && !strtemp.empty()) {
                rfharmonic = atof(strtemp.c_str());
                RFflag = 1;
            } else if (name == "MOMENTUM" && !strtemp.empty()) {
                momentum = atof(strtemp.c_str());
            } else if (name == "PLOTFLAG" && !strtemp.empty()) {
                plotflag = strtemp;
            } else if (name == "NAME") {
                break;
            } else {
              //  getline(entree, strtemp);
            }

        }
    }
    entree.close();
    
   

    //reads optics file
    extractOpticsParameters(opticsFile, beamflag);


    //order of the headers in collimatorfile: NAME,ANGLE,LENGTH,PHASE,NSIG,MATERIAL,METHOD

    ifstream entree1;
    vector <Collimator*> temp;

    entree1.open(inputfile.c_str());

    if (entree1.fail()) {
        cerr << "Warning: problem reading the second part of the collimator file!" << endl;
    } else {

        string NAME, MATERIAL, METHOD, strtemp1,ws1;
	string delimiter1 = ",";
	size_t  pos=0;
        
	double ANGLE=0.0, LENGTH=0.0;
        int PHASE=1;
        double NSIG2=0.0;


        getline(entree1, NAME, ',');
        while (NAME != "NAME") {
            getline(entree1, strtemp1);
            getline(entree1, NAME, ',');
        }
        getline(entree1, strtemp1);

       // while (!entree1.eof()) {
	  while (entree1 >> ws1) {
	    
	    pos = ws1.find(delimiter1);
	    NAME = ws1.substr(0,pos);
	    ws1.erase(0,pos+delimiter1.size());
	    
	    
	    pos = ws1.find(delimiter1);
	    strtemp1 = ws1.substr(0,pos);
	    ANGLE = atof(strtemp1.c_str());
	    ws1.erase(0,pos+delimiter1.size());
	    
	   
	    pos = ws1.find(delimiter1);
	    strtemp1 = ws1.substr(0,pos);
	    LENGTH = atof(strtemp1.c_str());
	    ws1.erase(0,pos+delimiter1.size());
	    
	    pos = ws1.find(delimiter1);
	    strtemp1 = ws1.substr(0,pos);
	     PHASE = atoi(strtemp1.c_str());
	    ws1.erase(0,pos+delimiter1.size());
	    
	    pos = ws1.find(delimiter1);
	    strtemp1 = ws1.substr(0,pos);
	    NSIG2 = atof(strtemp1.c_str());
	    ws1.erase(0,pos+delimiter1.size());
	    
	    
	    pos = ws1.find(delimiter1);
	    MATERIAL = ws1.substr(0,pos);
	    ws1.erase(0,pos+delimiter1.size());
	    
	    METHOD = ws1;
	    ws1.erase(0,ws1.size());
        
         

// 	    cout << NAME << endl;
// 	    cout << ANGLE << endl;
// 	    cout << LENGTH << endl;
// 	    cout << PHASE << endl;
// 	    cout << NSIG2 << endl;
// 	    cout << MATERIAL << endl;
// 	    cout << METHOD << endl;
// 	    

            for (int i(0); i < nbre_elts; ++i) {
	      
// 	      if(i==nbre_elts-1)
// 	      cout << nbre_elts - 1 << endl;
	      
                if (lat.reseau[i]->NAME == NAME) {
		  
		 // cout << "NAME is " << NAME << "i = " << i << endl; 
                    //we skip collimators that are not found in the optics file and collimators with names ending in the wrong letter
                    if ((atoi(&NAME[NAME.size() - 1]) == beamflag) || (NAME == "TCTV.4R2.B2") || (NAME == "TCTV.4R8.B2") || (NAME == "TCTV.4L2.B1") || (NAME == "TCTV.4L8.B1")) { //attention, il faut peut etre egalement supprimer ces elements dans le reseau
                       
                        lat.ips.push_back(i);

                        if (NAME.substr(0, 3) == "TCI") {
                            lat.ii.push_back(i);
                        } else  if ((NAME.substr(0, 3) == "TCP") || (NAME.substr(0, 7) == "CRYSTAL") || (NAME.substr(0, 5) == "TCRYO")) {
                            lat.ip.push_back(i);
                        } else  if (NAME.substr(0, 3) == "TCS") {
                            lat.is.push_back(i);
                        } else if (NAME.substr(0, 3) == "TCT") {
                            lat.it.push_back(i);
                        } else if (NAME.substr(0, 4) == "TCLA") {
                            lat.itcla.push_back(i);
                        }else
			  cout << "Simulation:readInputFile(): Warning! The name of the collimator ("<<
			           NAME <<") does not start with 'TCI', 'TCP', 'TCRYO', 'CRYSTAL', 'TCS', 'TCT'  or 'TCLA'"<<endl;;


                        if (METHOD == "standard") {

                            StandardCollimator stdcolli(*lat.reseau[i], ANGLE, NSIG2, METHOD, MATERIAL);
                            temp.push_back(new StandardCollimator(stdcolli));

                        } else if (METHOD == "magnetic") {

                            MagneticCollimator magnetcolli(*lat.reseau[i], ANGLE, NSIG2, METHOD, MATERIAL);
                            temp.push_back(new MagneticCollimator(magnetcolli));

                        } else if (METHOD == "fluka") {
#if defined(FLUKA)
                            RunWithFluka = 1;
                            FlukaCollimator flukacolli(*lat.reseau[i], ANGLE, NSIG2, METHOD);
                            temp.push_back(new FlukaCollimator(flukacolli));
#else
                            die("Fluka coupling not supported in this version, please compile with the appropriate flags");
#endif
                        } else if (METHOD == "crystal") {
                            RunWithCrystal = 1;
                            CrystalCollimator crystalcolli(*lat.reseau[i], ANGLE, NSIG2, METHOD);
                            temp.push_back(new CrystalCollimator(crystalcolli));
                        } else {

                            cerr << "Warning: unknown method (" << METHOD << ") for the collimator " << NAME << endl;
                        }
                    }
                }
            }
        }
    }

    entree1.close();

    sort(lat.ip.begin(), lat.ip.end());
    sort(lat.ii.begin(), lat.ii.end());
    sort(lat.is.begin(), lat.is.end());
    sort(lat.it.begin(), lat.it.end());
    sort(lat.itcla.begin(), lat.itcla.end());

    //sorting all the information according to position along the ring

    sort(lat.ips.begin(), lat.ips.end());

    vector <int> indice;
    vector <double> pos, pos1;

    for (int k(0); k < temp.size(); ++k) {
        pos.push_back(temp[k]->S);
        pos1.push_back(temp[k]->S);
    }

    sort(pos.begin(), pos.end());

    for (int j(0); j < pos.size(); ++j) {
        int m(0);
        for (int k(0); k < pos1.size(); ++k) {
            if (pos[j] == pos1[k]) {
                indice.push_back(m);
                pos1[k] = -1;
                break;
            } else {
                ++m;
            }
        }
    }

    for (int j(0); j < temp.size(); ++j) {
        if (temp[indice[j]]->method == "standard") {

            lat.addCollimator(temp[indice[j]]);
            lat.resColli[j]->crossSectionPath = crosssectionspath + '/' + lat.resColli[j]->material + '/';

        } else if (temp[indice[j]]->method == "magnetic") {

            lat.addCollimator(temp[indice[j]]);
            lat.resColli[j]->crossSectionPath = crosssectionspath + '/' + lat.resColli[j]->material + '/';
            lat.resColli[j]->Bmax = Bmax;
            lat.resColli[j]->thicknessMagneticField = thicknessmagneticfield;
            lat.resColli[j]->energyPerIon = energyPerIon;
            lat.resColli[j]->mass = mass;
            lat.resColli[j]->deltaGap = deltaGap;

        } else if (temp[indice[j]]->method == "fluka") {

            lat.addCollimator(temp[indice[j]]);

        } else if (temp[indice[j]]->method == "crystal") {

            lat.addCollimator(temp[indice[j]]);

            ifstream readinfocrystal;
            string inputtemp;
            inputtemp = inputpath + "crystalinfo.csv";
            readinfocrystal.open(inputtemp.c_str());

            if (readinfocrystal.fail()) {
                cerr << "Warning: problem reading the file crystalinfo.csv!" << endl;
            } else {
                string temp1;
                getline(readinfocrystal, temp1, ',');
                while (!readinfocrystal.eof()) {
                    if (temp[indice[j]]->NAME == temp1) {
                        getline(readinfocrystal, temp1, ',');
                        lat.resColli[j]->C_orient = atoi(temp1.c_str());
                        getline(readinfocrystal, temp1, ',');
                        lat.resColli[j]->IS = atoi(temp1.c_str());
			//added feature, to define the install location of the crystal, then 
			//get the correct transformation between the beam frame & crystal frame.
			getline(readinfocrystal, temp1, ',');
                        lat.resColli[j]->Mirror = atoi(temp1.c_str());
			
                        getline(readinfocrystal, temp1, ',');
                        lat.resColli[j]->C_xmax = atof(temp1.c_str());
                        getline(readinfocrystal, temp1, ',');
                        lat.resColli[j]->C_ymax = atof(temp1.c_str());
                        getline(readinfocrystal, temp1, ',');
                        lat.resColli[j]->Cry_length = atof(temp1.c_str());
                        getline(readinfocrystal, temp1, ',');
                        lat.resColli[j]->Rcurv = atof(temp1.c_str());
                        getline(readinfocrystal, temp1, ',');
                        lat.resColli[j]->C_rotation = atof(temp1.c_str());
                        getline(readinfocrystal, temp1, ',');
                        lat.resColli[j]->C_aperture = atof(temp1.c_str());
                        getline(readinfocrystal, temp1, ',');
                        lat.resColli[j]->C_offset = atof(temp1.c_str());
                        getline(readinfocrystal, temp1, ',');
                        lat.resColli[j]->C_tilt = atof(temp1.c_str());
                        getline(readinfocrystal, temp1);
                        lat.resColli[j]->Cry_tilt = atof(temp1.c_str());
                    } //else {
		      //cout << "Simulation::readInputFile(): Warning: the crystal ("
		       //    << temp1 <<") is not defined in the collimator file" <<endl;
                      //  getline(readinfocrystal, temp1);
                    //}
                    getline(readinfocrystal, temp1, ',');
                }
                readinfocrystal.close();
            }

        } else {

            cerr << "Warning: problem to construct resColli " << endl;
        }
    }

    for (int j(0); j < lat.resColli.size(); ++j) {
        lat.resColli[j]->tcang = lat.resColli[j]->tcang * M_PI / 180;
    }

    vector <double> sx, sy;

    //In this calculation we do not take into account that there is variation in the momentum! In that case the formulas would be:
    //sx=nsig*sqrt(bx*emix+dx^2*sigdpp^2) and sy=nsig*sqrt(by*emiy+dy^2*sigdpp^2)

    for (int k(0); k < lat.resColli.size(); ++k) {

        sx.push_back(lat.resColli[k]->nsig * sqrt(lat.resColli[k]->BETX * emix));
        sy.push_back(lat.resColli[k]->nsig * sqrt(lat.resColli[k]->BETY * emiy));
    }

    double b[lat.resColli.size()];

    for (int j(0); j < lat.resColli.size(); ++j) {
        if (sin(lat.resColli[j]->tcang) == 0) {
            b[j] = sx[j];
        } else if (sin(lat.resColli[j]->tcang) > 0) { //collimator is straight
            b[j] = sin(lat.resColli[j]->tcang) * sqrt(sy[j] * sy[j] + sx[j] * sx[j] / (tan(lat.resColli[j]->tcang) * tan(lat.resColli[j]->tcang)));
        } else {
            b[j] = 0;
        }
    }

    if (wecolli >= 0) { //rotate collimator around face
        for (int k(0); k < lat.resColli.size(); ++k) {
            lat.resColli[k]->hgap = b[k];
            lat.resColli[k]->hgap2 = b[k] + lat.resColli[k]->L * wecolli;
        }
    } else {//rotate collimator around end
        for (int k(0); k < lat.resColli.size(); ++k) {
            lat.resColli[k]->hgap = b[k] + lat.resColli[k]->L * (-wecolli);
            lat.resColli[k]->hgap2 = b[k];
        }
    }

    for (int k(0); k < lat.resColli.size(); ++k) {
        lat.resColli[k]->phi = (lat.resColli[k]->hgap2 - lat.resColli[k]->hgap) / lat.resColli[k]->L; //Calculate absolute angle of collimator; installation angle
    }


    for (int n(0); n < lat.ips.size(); ++n) {
        for (int p(0); p < lat.ip.size(); ++p) {
            if (lat.ips[n] == lat.ip[p]) {
                lat.ipcoll.push_back(n);
            }
        }
    }


//     for(int k(0); k < lat.resColli.size(); ++k){
//      cout << "test.....";
//       cout << lat.resColli[k]->hgap <<"    " <<lat.resColli[k]->hgap2 <<endl;  
//     
//     }
      
    lat.freqrf = freqrf;
    lat.rfvoltage = rfvoltage;
    lat.rfharmonic = rfharmonic;
    lat.momentum = momentum;

}



void Simulation::run(string inputfile, string outputpath)
{

    cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
    cout << "++++++++++++++++++++++++++++++++++++++   NEW SIMULATION   +++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
    cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;


    double Apr=0.0, Zpr=0.0, mass=0.0, energyPerIon=0.0, emix=0.0, emiy=0.0, sigdpp=0.0,
    r1r2skin=0.0, blowup2=0.0, thicknessmagneticfield=0.0, Bmax=0.0, deltaGap=0.0, 
    gamma=0.0, betgam=0.0, W=0.0, PlossPb=0.0, freqrf=0.0, rfvoltage=0.0, rfharmonic=0.0, 
    wecolli=0.0, nsigi=0.0, momentum=0.0;
    long int npart=0L, nparti=0L, nrev1=0L;
    int kbunch=0, taubeam=0, nrev2=0, blowupperiod=0, scaleorbit=0,
    beamflag=1, stopLinearTrackingNo=0, RunWithFluka=0, RunWithCrystal=0, RunningFlag=1,
    outcoord=0, idpart=0, idelt=0, RFflag=0;
    string partdistr="", stoplineartracking="", crosssectionspath="", opticsFile="", plotflag="", inputpath="";

    int indication(1);
    outcoord = 0;

    RunWithFluka = 0;
    RunWithCrystal = 0;
    RFflag = 0;
    
     cout <<" "<<endl;
      cout <<"********************************"<<endl;
    cout << "READING OF THE INPUT FILES." << endl;

    //getting information about the optics, apertures and collimators from the input file
    try {
        readInputFile(inputfile, Apr, Zpr, mass, energyPerIon, emix, emiy, npart, kbunch, sigdpp, taubeam, nparti, partdistr, r1r2skin, nrev1, nrev2, blowup2, blowupperiod, stoplineartracking, scaleorbit, wecolli, nsigi, opticsFile, thicknessmagneticfield, Bmax, deltaGap, beamflag, RunWithFluka, RunWithCrystal, freqrf, rfvoltage, rfharmonic, RunningFlag, idpart, idelt, outcoord, momentum, plotflag, inputpath, RFflag);
    }


    catch (int erreur) {
        cerr << "Error 1: no aperture infos!!" << endl;
        cerr << "The program has ended without stopping the fluka server. You have to kill it by yourself (pkill flukaserver)." << endl;
    }

    momentum = sqrt(energyPerIon * energyPerIon - (mass * mass));
    momentum = momentum * 1e09;

    if ((RunningFlag != 1) && (RunningFlag != 2) && (RunningFlag != 3) && (RunningFlag != 4)) {
        cerr << "Warning: Running Flag must be equal to 1, 2, 3 or 4!!" << endl;
        cerr << "Check it in the collimator file, if it as no RUNNINGFLAG parameter, you must add it" << endl;
        return;
    }

    gamma = energyPerIon / mass;
    betgam = sqrt(gamma * gamma - 1);

    //The beginning of the tracking with 'trackensemblechrom' is linear. We decide when we start doing the nonlinear tracking.

    for (int i(0); i < lat.reseau.size(); ++i) {
        if (stoplineartracking == lat.reseau[i]->NAME) {
            stopLinearTrackingNo = i;
            break;
        }
    }

    if (stoplineartracking.size() == 0) {

        cerr << "Warning: the element where we want to stop the linear tracking does not exist!" << endl;
    }

    W = npart;
    W = W * kbunch;
    W = W * energyPerIon;
    W = W * 1000000000;
    W = W * 1.602e-19; //total energy of beam
    PlossPb = W / taubeam; // average power loss of beam

    lat.setsize_res(lat.reseau.size());

    if ((RunWithCrystal == 1) && ((Apr != 1) || (Zpr != 1))) {
        cerr << "Warning: Crystal collimation can only be used with protons!!" << endl;
        return;
    }

    int i0(0);

   // cout << "GENERATION OF THE BUNCH OF PARTICLES." << endl;

    //===============================================================================================================================================================================================
    //THE FOLLOWING PART CAN BE COMMENTED/UNCOMMENTED TO COMPARE EXACT VALUES WITHOUT RANDOMNESS WITH ICOSIM (SEE THE FILE RANDOMNESS.TXT FOR MORE EXPANATION ON WHAT TO DO TO ELIMINATE RANDOMNESS)

    //WE ALSO HAVE THE POSSIBILITY HERE TO READ THE PARTICLES FROM A FILE WITH THE FUNCTION readParticle
    //===============================================================================================================================================================================================

    if (RunningFlag == 2) {

        srand(time(0));
	cout <<" "<<endl;
      cout <<"********************************"<<endl;
        cout << "RunningFlag = " <<  RunningFlag << endl;
	
        readParticle(Apr, Zpr, inputfile);

        //we set the id of the particles defined in the bunch
        for (int y(0); y < bunch.size(); ++y) {
            bunch[y].setidentification(y+1);
        }

    } else if (RunningFlag == 1) {
      cout <<" "<<endl;
      cout <<"********************************"<<endl;
      cout << "RunningFlag = " <<  RunningFlag << endl;
      cout << "GENERATION OF THE BUNCH OF PARTICLES." << endl;
        genbunch(i0, nparti, partdistr, r1r2skin, nsigi * nsigi * emix, nsigi * nsigi * emiy, sigdpp, lat.reseau[i0]->BETX, lat.reseau[i0]->ALFX, lat.reseau[i0]->DX, lat.reseau[i0]->DPX, lat.reseau[i0]->BETY, lat.reseau[i0]->ALFY, lat.reseau[i0]->DY, lat.reseau[i0]->DPY, nsigi, inputfile);

        //we set the id of the particles
        for (int y(0); y < bunch.size(); ++y) {
            bunch[y].setidentification(y+1);
        }

    }

    /*Particle p1(-9.62058e-07, -0.000153021, -4.97476e-05, -4.80766e-05, -8.21051e-05, 0, 208, 82,  -0.00012159388136310807);
    Particle p2(-2.29177e-05, -0.000135618, -6.02201e-05, 3.8913e-05, -6.28236e-05, 0, 208, 82,  -6.569201361245118e-05);
    Particle p3(1e-06, -0.00015, -5e-05, 2e-05, -7e-05, 0, 208, 82, -7e-05);
    Particle p4(2e-05, -0.00012, 2e-07, 4e-06, 5e-05, 0, 208, 82, 5e-05);
    Particle p5(-3e-05, -0.00016, 3e-05, -2e-05, -3e-06, 0, 208, 82, -3e-06);
    Particle p6(-1e-06, -0.00013, -2e-05, -2e-05, 2e-05, 0, 208, 82, 2e-05);
    Particle p7(1.5e-05, -0.00015, -3e-06, 1.5e-05, 6e-05, 0, 208, 82, 6e-05);
    Particle p8(8e-06, -0.00013, 4e-06, 2.3e-05, -6e-05, 0, 208, 82, -6e-05);
    Particle p9(-4e-05, -0.00015, 5e-05, 4e-05, 5e-05, 0, 208, 82, 5e-05);
    Particle p10(5e-05, -0.00016, 6e-07, -5e-05, -3e-05, 0, 208, 82, -3e-05);



    bunch.push_back(p1);
    bunch.push_back(p2);
    bunch.push_back(p3);
    bunch.push_back(p4);
    bunch.push_back(p5);
    bunch.push_back(p6);
    bunch.push_back(p7);
    bunch.push_back(p8);
    bunch.push_back(p9);
    bunch.push_back(p10);*/


    //we set the id of the particles
    /*for(int y(0); y<bunch.size(); ++y){
      bunch[y].setidentification(y);
      }*/

    //================================================================= END OF TEMP. MODIFICATIONS ===========================================================================================
    int particlehit=0;

    if ((RunningFlag == 1) || (RunningFlag == 2)) {
      cout <<" "<<endl;
      cout <<"********************************"<<endl;
        cout << "FIRST TRACKING." << endl;
        cout << "Linear tracking of " <<nrev1<< " turns."<<endl;
	cout << "To simulate halo generation process:" <<endl;
	cout << "The 4D coordinates [x, xp, y, yp] of particles blow up by a coefficient "<<sqrt(blowup2)<<" in every "<<blowupperiod<<" turns"<<endl;  
        //We track the particles linearly between the primary collimators, to find the particles hit on the collimators in "nrev1" turns
	// to save the simulation time in the following nonlinear tracking...
        lat.trackensemblelinearnew(bunch, bunchhit, nrev1, blowup2, blowupperiod);
        particlehit = bunchhit.size();

        for (int i(0); i < bunchhit.size(); ++i) {
            bunchhit[i].Ap0 = Apr;
            bunchhit[i].Zp0 = Zpr;
        }
    }


    if (RunningFlag == 4) {
        srand(time(0));
	cout <<" "<<endl;
      cout <<"********************************"<<endl;
	cout << "RunningFlag = " <<  RunningFlag << endl;
        readParticle(Apr, Zpr, inputfile);
    } else if (RunningFlag == 3) {
      cout <<" "<<endl;
      cout <<"********************************"<<endl;
      cout << "RunningFlag = " <<  RunningFlag << endl;
      cout << "GENERATION OF THE BUNCH OF PARTICLES." << endl;
        genbunch(i0, nparti, partdistr, r1r2skin, nsigi * nsigi * emix, nsigi * nsigi * emiy, sigdpp, lat.reseau[i0]->BETX, lat.reseau[i0]->ALFX, lat.reseau[i0]->DX, lat.reseau[i0]->DPX, lat.reseau[i0]->BETY, lat.reseau[i0]->ALFY, lat.reseau[i0]->DY, lat.reseau[i0]->DPY, nsigi, inputfile);
    }

    if ((RunningFlag == 3) || (RunningFlag == 4)) {
        for (int y(0); y < bunch.size(); ++y) {
	 
            bunch[y].setidentification(y+1);
// 	     cout << "test....... y = "<< y << endl;
// 	     cout << "id = " << bunch[y].getidentification() <<endl;
         }

        for (int k(0); k < bunch.size(); ++k) {
            bunchhit.push_back(bunch[k]);
        }
        particlehit = bunchhit.size();

        for (int i(0); i < bunchhit.size(); ++i) {
            bunchhit[i].Ap0 = Apr;
            bunchhit[i].Zp0 = Zpr;
        }
    }
    //#################################################################################################################################

#if defined(FLUKA)
    //if we have a Fluka element
    if (RunWithFluka == 1) { //connection to Fluka

        int port = PORT;

        lat.conn = flukaio_connect("pcen33148", port);//Warning: the hostname (pcen33148) must be changed to yours. Your hostname can be find in the file network.nfo under the directory ICOSIM++/<name_of_run.nber>/1

        if (lat.conn == NULL) {
            die("Error connecting to server");
        }

        cout << "Connected to server" << endl;
    }
#endif


    if (bunchhit.size() == 0) { //no hit during the first tracking
        string outputfile;
	
        cout << endl;
	cout << "After "<<nrev1 << " turns of linear tracking:"<<endl;
        cout << "all the particles go througt the accelerator. No particle hits on the primary collimators!!!" << endl;

#if defined(FLUKA)
        if (RunWithFluka == 1) { //if we have a Fluka Collimator, we disconnect

            cout << "End of the connection." << endl;

            int n;
            flukaio_message_t msg;
            n = flukaio_send_eoc(lat.conn);
            if (n < 0) {
                die("Error occurred sending");
            }

            n = flukaio_wait_message(lat.conn, &msg);
            if (n == -1) {
                die("Server timeout when waiting end of computation");
            }
            if (msg.type != N_END) {
                die("Unexpected message received");
            }

            flukaio_disconnect(lat.conn);
            lat.conn = NULL;
        }
#endif

        return;


    }


    //We take the particles hitting collimators during the previous tracking (we take their coordinates at the beginning of the last turn before the hit), and do a  preciser tracking
    cout <<" "<<endl;
      cout <<"********************************"<<endl;
    cout << "SECOND TRACKING (LINEAR IN THE BEGINNING OF THE FIRST REVOLUTION)" << endl;
     cout <<"Nonlinear tracking."<<endl;
     
    vector <double>  Aphito, Zphito, hitso;
    vector <int>  nhitcollio, nhitspoilero;
    vector <vector <double> > xco, yco;
    int irev(0);
    int nonlinflag(0);
    double attr1(0);
    nonlinflag = 0;//the second tracking is linear

    lat.trackensemblechrom(bunchhit, irev, i0, stopLinearTrackingNo, Apr, Zpr, wecolli, betgam, nonlinflag, scaleorbit, attr1, idpart, idelt, outcoord, plotflag, xco, yco, outputpath, RFflag, indication);

    if (bunchhit.size() == 0) {
        cout << "No more particles!!" << endl;
#if defined(FLUKA)
        if (RunWithFluka == 1) {

            cout << "End of the connection." << endl;

            int n;
            flukaio_message_t msg;
            n = flukaio_send_eoc(lat.conn);
            if (n < 0) {
                die("Error occurred sending");
            }

            n = flukaio_wait_message(lat.conn, &msg);
            if (n == -1) {
                die("Server timeout when waiting end of computation");
            }
            if (msg.type != N_END) {
                die("Unexpected message received");
            }

            flukaio_disconnect(lat.conn);
            lat.conn = NULL;
        }
#endif

        return;
    }



    //tracking the particles to the end of the first revolution
     cout <<" "<<endl;
      cout <<"********************************"<<endl;
    cout << "THIRD TRACKING, END OF THE FIRST REVOLUTION." << endl;

    int im;

    im = lat.reseau.size() - 1;
    irev = 1;
    nonlinflag = 1;//this tracking is non linear

    lat.trackensemblechrom(bunchhit, irev, stopLinearTrackingNo, im, Apr, Zpr, wecolli, betgam, nonlinflag, scaleorbit, attr1, idpart, idelt, outcoord, plotflag, xco, yco, outputpath, RFflag, indication);

    for (int i(0); i < lat.hits.size(); ++i) {
        hitso.push_back(lat.hits[i]);//vector of s values where a particle hits the aperture
    }

    for (int i(0); i < lat.Aphit.size(); ++i) {
        Aphito.push_back(lat.Aphit[i]);//the mass number of the particles when they hit the aperture
    }

    for (int i(0); i < lat.Zphit.size(); ++i) {
        Zphito.push_back(lat.Zphit[i]);//the charge of the particles when they hit the aperture
    }


    for (int i(0); i < lat.nhitcolli.size(); ++i) {
        nhitcollio.push_back(lat.nhitcolli[i]);//nhitcolli(icop) gives the number of particles that are absorbed at collimator number 'icop'
    }

    for (int i(0); i < lat.nhitspoiler.size(); ++i) {
        nhitspoilero.push_back(lat.nhitspoiler[i]);//same as nhitcollio, but for ion spoilers
    }


    vector <int> asumrem, asumhitcolli, asumhitspoiler, asumhits;


    asumhitcolli.push_back(0);//how many particles that have been absorbed in a collimator, as a function of the number of revolutions
    asumhits.push_back(0);//number of particles that have hitted the aperture, as a function of the number of revolutions
    asumrem.push_back(particlehit);//how many particles there are left in the beam after each revolution
    asumhits.push_back(lat.hits.size());
    lat.hits.clear();

    int count1(0);
    int count(0);

    for (int i(0); i < nhitcollio.size(); ++i) {
        count = count + nhitcollio[i];
    }

    count1 = count;

    asumhitcolli.push_back(count);

    lat.nhitcolli.clear();

    count = 0;

    for (int i(0); i < lat.nhitspoiler.size(); ++i) {
        count = count + lat.nhitspoiler[i];
    }

    count1 = count1 + count;

    asumhitspoiler.push_back(count);//how many particles that have been absorbed by an ion spoiner, as a function of the number of revolutions
    lat.nhitspoiler.clear();
    asumrem.push_back(bunchhit.size());

    if (bunchhit.size() == 0) { //if we have no more particle, we prepare a final output
        cout << "No more particles!!" << endl;
        string outputfile;
        string inputtemp;
        inputtemp = inputfile.substr(10, inputfile.size() - 1);
        outputfile = outputpath + "/output.txt";

        writeoutput(outputfile, emix, emiy, Apr, Zpr, nparti, beamflag, PlossPb, i0, im, npart, kbunch, asumrem, asumhitcolli, asumhits, taubeam, hitso, nhitcollio, Aphito, Zphito, i0, stopLinearTrackingNo, xco, yco, plotflag, outputpath, inputpath);

#if defined(FLUKA)
        if (RunWithFluka == 1) {

            cout << "End of the connection." << endl;

            int n;
            flukaio_message_t msg;
            n = flukaio_send_eoc(lat.conn);
            if (n < 0) {
                die("Error occurred sending");
            }

            n = flukaio_wait_message(lat.conn, &msg);
            if (n == -1) {
                die("Server timeout when waiting end of computation");
            }
            if (msg.type != N_END) {
                die("Unexpected message received");
            }

            flukaio_disconnect(lat.conn);
            lat.conn = NULL;
        }
#endif

        return;
    }

    if (asumrem[asumrem.size() - 1] == 0) {

        cout << "No more particle!!" << endl;

        string outputfile;
        string inputtemp;
        inputtemp = inputfile.substr(10, inputfile.size() - 1);
        outputfile = outputpath + "/output.txt";

        writeoutput(outputfile, emix, emiy, Apr, Zpr, nparti, beamflag, PlossPb, i0, im, npart, kbunch, asumrem, asumhitcolli, asumhits, taubeam, hitso, nhitcollio, Aphito, Zphito, i0, stopLinearTrackingNo, xco, yco, plotflag, outputpath, inputpath);

#if defined(FLUKA)
        if (RunWithFluka == 1) {

            cout << "End of the connection." << endl;

            int n;
            flukaio_message_t msg;
            n = flukaio_send_eoc(lat.conn);
            if (n < 0) {
                die("Error occurred sending");
            }

            n = flukaio_wait_message(lat.conn, &msg);
            if (n == -1) {
                die("Server timeout when waiting end of computation");
            }
            if (msg.type != N_END) {
                die("Unexpected message received");
            }

            flukaio_disconnect(lat.conn);
            lat.conn = NULL;
        }
#endif

        return;
    }



    lat.nhitspoiler.clear();


    //nonlinear tracking the particles through the machine
     cout <<" "<<endl;
      cout <<"********************************"<<endl;
     cout << "FOURTH TRACKING, TRACKING THROUGHT THE MACHINE." << endl;
     cout << "Nonlinear tracking the particles through element by element in the machine."<<endl;

    for (int i(1); i < nrev2; ++i) {

        cout << i + 1 << "th revolution." << endl;


        bool empty;
        empty = true;

        for (int j(0); j < bunchhit.size(); ++j) {
            if (bunchhit[j].inabs == 1) {
                empty = false;
            }
        }

        if (empty == true) {

            cout << "No more particle! The tracking through the machine ends." << endl;

            string outputfile;
            string inputtemp;
            inputtemp = inputfile.substr(10, inputfile.size() - 1);
            outputfile = outputpath + "/output.txt";


            writeoutput(outputfile, emix, emiy, Apr, Zpr, nparti, beamflag, PlossPb, i0, im, npart, kbunch, asumrem, asumhitcolli, asumhits, taubeam, hitso, nhitcollio, Aphito, Zphito, i0, stopLinearTrackingNo, xco, yco, plotflag, outputpath, inputpath);

#if defined(FLUKA)
            if (RunWithFluka == 1) {

                cout << "End of the connection." << endl;

                int n;
                flukaio_message_t msg;
                n = flukaio_send_eoc(lat.conn);
                if (n < 0) {
                    die("Error occurred sending");
                }

                n = flukaio_wait_message(lat.conn, &msg);
                if (n == -1) {
                    die("Server timeout when waiting end of computation");
                }
                if (msg.type != N_END) {
                    die("Unexpected message received");
                }

                flukaio_disconnect(lat.conn);
                lat.conn = NULL;
            }
#endif

            return;
        }

        lat.trackensemblechrom(bunchhit, i, i0, im, Apr, Zpr, wecolli, betgam, nonlinflag, scaleorbit, attr1, idpart, idelt, outcoord, plotflag, xco, yco, outputpath, RFflag, indication);


        for (int m(0); m < lat.hits.size(); ++m) {
            hitso.push_back(lat.hits[m]);
        }

        for (int m(0); m < lat.Aphit.size(); ++m) {
            Aphito.push_back(lat.Aphit[m]);
        }

        for (int m(0); m < lat.Zphit.size(); ++m) {
            Zphito.push_back(lat.Zphit[m]);
        }

        for (int m(0); m < lat.nhitcolli.size(); ++m) {
	    cout << "m = " << m << "nhitcollio[m] is: " << nhitcollio[m] << " lat.nhitcolli[m] is:  " << lat.nhitcolli[m] << endl; 
            nhitcollio[m] = nhitcollio[m] + lat.nhitcolli[m];
        }

        for (int m(0); m < lat.nhitspoiler.size(); ++m) {
            nhitspoilero[m] = nhitspoilero[m] + lat.nhitspoiler[m];
	    
        }

        asumhits.push_back(asumhits[asumhits.size() - 1] + lat.hits.size());
        lat.hits.clear();
        lat.Aphit.clear();
        lat.Zphit.clear();

        int count1(0);
        int count(0);

        for (int l(0); l < nhitcollio.size(); ++l) {
            count = count + nhitcollio[l];
        }

        count1 = count;
        asumhitcolli.push_back(count);

        lat.nhitcolli.clear();

        count = 0;

        for (int l(0); l < lat.nhitspoiler.size(); ++l) {
            count = count + lat.nhitspoiler[l];
        }

        count1 = count1 + count;

        asumhitspoiler.push_back(count);
        asumrem.push_back(bunchhit.size());
        lat.nhitspoiler.clear();

        if (bunchhit.size() == 0) {
            cout << "No more particles!!" << endl;

            string outputfile;
            string inputtemp;
            inputtemp = inputfile.substr(10, inputfile.size() - 1);
            outputfile = outputpath + "/output.txt";

            writeoutput(outputfile, emix, emiy, Apr, Zpr, nparti, beamflag, PlossPb, i0, im, npart, kbunch, asumrem, asumhitcolli, asumhits, taubeam, hitso, nhitcollio, Aphito, Zphito, i0, im, xco, yco, plotflag, outputpath, inputpath);

#if defined(FLUKA)
            if (RunWithFluka == 1) {

                cout << "End of the connction." << endl;

                int n;
                flukaio_message_t msg;
                n = flukaio_send_eoc(lat.conn);
                if (n < 0) {
                    die("Error occurred sending");
                }

                n = flukaio_wait_message(lat.conn, &msg);
                if (n == -1) {
                    die("Server timeout when waiting end of computation");
                }
                if (msg.type != N_END) {
                    die("Unexpected message received");
                }

                flukaio_disconnect(lat.conn);
                lat.conn = NULL;
            }
#endif

            return;
        }

        if (asumrem[i] == 0) {
            string outputfile;
            string inputtemp;
            inputtemp = inputfile.substr(10, inputfile.size() - 1);
            outputfile = outputpath + "/output.txt";

            writeoutput(outputfile, emix, emiy, Apr, Zpr, nparti, beamflag, PlossPb, i0, im, npart, kbunch, asumrem, asumhitcolli, asumhits, taubeam, hitso, nhitcollio, Aphito, Zphito, i0, im, xco, yco, plotflag, outputpath, inputpath);

#if defined(FLUKA)
            if (RunWithFluka == 1) {

                cout << "End of the connection." << endl;

                int n;
                flukaio_message_t msg;
                n = flukaio_send_eoc(lat.conn);
                if (n < 0) {
                    die("Error occurred sending");
                }

                n = flukaio_wait_message(lat.conn, &msg);
                if (n == -1) {
                    die("Server timeout when waiting end of computation");
                }
                if (msg.type != N_END) {
                    die("Unexpected message received");
                }

                flukaio_disconnect(lat.conn);
                lat.conn = NULL;
            }
#endif

            return;
        }

    }

    string outputfile;
    string inputtemp;
    inputtemp = inputfile.substr(10, inputfile.size() - 1);
    outputfile = outputpath + "/output.txt";

    writeoutput(outputfile, emix, emiy, Apr, Zpr, nparti, beamflag, PlossPb, i0, im, npart, kbunch, asumrem, asumhitcolli, asumhits, taubeam, hitso, nhitcollio, Aphito, Zphito, i0, im, xco, yco, plotflag, outputpath, inputpath);

#if defined(FLUKA)
    if (RunWithFluka == 1) {

        cout << "End of the connection." << endl;

        int n;
        flukaio_message_t msg;
        n = flukaio_send_eoc(lat.conn);
        if (n < 0) {
            die("Error occurred sending");
        }

        n = flukaio_wait_message(lat.conn, &msg);
        if (n == -1) {
            die("Server timeout when waiting end of computation");
        }
        if (msg.type != N_END) {
            die("Unexpected message received");
        }

        flukaio_disconnect(lat.conn);
        lat.conn = NULL;
    }
#endif

}


void Simulation::extractOpticsParameters(string opticsFile, const int& beamflag)
{

//for the moment, ICOSIM++ cannot run without aperture parameters, so JohnJaperflag is fixed to 1; the variable is still here to facilitate further developpment
    int JohnJaperFlag=1;

    defineParameters(opticsFile, beamflag, JohnJaperFlag);

    bool apertureIsZero(false);
    int temp1 = 0;
    int temp2 = 0;
    int temp3 = 0;
     
    
    if (JohnJaperFlag == 1) { //JohnJaperflag=1 if we have information about the aperture

        for (int w(0); w < nbre_elts; ++w) {
            if (lat.reseau[w]->APERTYPE == "RECTELLIPSE") {
                temp1 = temp1 + 1;
                if ((lat.reseau[w]->APER_1 == 0) && (lat.reseau[w]->APER_2 == 0) && (lat.reseau[w]->APER_3 == 0) && (lat.reseau[w]->APER_4 == 0)) {
                    temp2 = temp2 + 1;
                } 
//                 else {
//                     break;
//                 }
            } 
//             else {
//                 break;
//             }
        }

        //If all apertures are RECTELLIPSES:
           //If all the aperture variables of all lattice elements are 0, we define them to be 1 instead.
        //  If an apeture variable is 0, it takes the value of the preceeding aperture variable.

        if (temp1 == nbre_elts) {
            if (temp2 == nbre_elts) {
                for (int w(0); w < nbre_elts; ++w) {
                    lat.reseau[w]->APER_1 = 1;
                    lat.reseau[w]->APER_2 = 1;
                    lat.reseau[w]->APER_3 = 1;
                    lat.reseau[w]->APER_4 = 1;
                }
            }
            //if some apertures of the first elements are 0, we define them equal to those of the element nearest of the end of the lattice with apertures not equal to 0
            if (lat.reseau[0]->APER_1 == 0) {
                apertureIsZero = true;
                temp3 = nbre_elts -1;
                if (lat.reseau[temp3]->APER_1 == 0) {
                    temp3 = temp3 - 1;
                }
                lat.reseau[0]->APER_1 = lat.reseau[temp3]->APER_1;
            }
            if (lat.reseau[0]->APER_2 == 0) {
                apertureIsZero = true;
                temp3 = nbre_elts -1;
                if (lat.reseau[temp3]->APER_2 == 0) {
                    temp3 = temp3 - 1;
                }
                lat.reseau[0]->APER_2 = lat.reseau[temp3]->APER_2;
            }
            if (lat.reseau[0]->APER_3 == 0) {
                apertureIsZero = true;
                temp3 = nbre_elts -1;
                if (lat.reseau[temp3]->APER_3 == 0) {
                    temp3 = temp3 - 1;
                }
                lat.reseau[0]->APER_3 = lat.reseau[temp3]->APER_3;
            }
            if (lat.reseau[0]->APER_4 == 0) {
                apertureIsZero = true;
                temp3 = nbre_elts - 1;
                if (lat.reseau[temp3]->APER_4 == 0) {
                    temp3 = temp3 - 1;
                }
                lat.reseau[0]->APER_4 = lat.reseau[temp3]->APER_4;
            }

            //if some other apertures are equal to 0, we define them equal to the preceding ones
            for (int p(1); p < nbre_elts; ++p) {
                if (lat.reseau[p]->APER_1 == 0) {
                    apertureIsZero = true;
                    lat.reseau[p]->APER_1 = lat.reseau[p - 1]->APER_1;
                }
                if (lat.reseau[p]->APER_2 == 0) {
                    apertureIsZero = true;
                    lat.reseau[p]->APER_2 = lat.reseau[p - 1]->APER_2;
                }
                if (lat.reseau[p]->APER_3 == 0) {
                    apertureIsZero = true;
                    lat.reseau[p]->APER_3 = lat.reseau[p - 1]->APER_3;
                }
                if (lat.reseau[p]->APER_4 == 0) {
                    apertureIsZero = true;
                    lat.reseau[p]->APER_4 = lat.reseau[p - 1]->APER_4;
                }
            }
        }

        //if the apertures are not only RECTANGLE:
        //if the aperture type is known, we just take the preceding apertures if we don't know the current ones (not the ideal solution if the aperture type changes);
        //if the aperture type is NONE, we also take the aperture type of the preceding element
        for (int r(0); r < nbre_elts; ++r) {
            if (lat.reseau[r]->APERTYPE != "NONE") {
                if ((lat.reseau[r]->APER_1 == 0) && (lat.reseau[r]->APER_2 == 0) && (lat.reseau[r]->APER_3 == 0) && (lat.reseau[r]->APER_4 == 0)) {
                    apertureIsZero = true;
                    lat.reseau[r]->APER_1 = 1;;
                    lat.reseau[r]->APER_2 = 1;
                    lat.reseau[r]->APER_3 = 1;
                    lat.reseau[r]->APER_4 = 1;
                }
            } else if (lat.reseau[r]->APERTYPE == "NONE") {
	      
	        if(r==0){
		  lat.reseau[r]->APER_1 = lat.reseau[nbre_elts]->APER_1;
		  lat.reseau[r]->APER_2 = lat.reseau[nbre_elts]->APER_2;
		  lat.reseau[r]->APER_3 = lat.reseau[nbre_elts]->APER_3;
		  lat.reseau[r]->APER_4 = lat.reseau[nbre_elts]->APER_4;
		  lat.reseau[r]->APERTYPE = lat.reseau[nbre_elts]->APERTYPE;
		}else{
                lat.reseau[r]->APER_1 = lat.reseau[r - 1]->APER_1;
                lat.reseau[r]->APER_2 = lat.reseau[r - 1]->APER_2;
                lat.reseau[r]->APER_3 = lat.reseau[r - 1]->APER_3;
                lat.reseau[r]->APER_4 = lat.reseau[r - 1]->APER_4;
                lat.reseau[r]->APERTYPE = lat.reseau[r - 1]->APERTYPE;
		}
            }
        }

        if (apertureIsZero == true) {
            cout << "At least one of the apertures in the file is 0." << endl;
        }

        //we compute the final apertures; different aperture type
        for (int i(0); i < lat.reseau.size(); ++i) {
            if (lat.reseau[i]->APERTYPE == "CIRCLE") {
                lat.reseau[i]->aperx = lat.reseau[i]->APER_1;
                lat.reseau[i]->apery = lat.reseau[i]->APER_1;
            } else if (lat.reseau[i]->APERTYPE == "ELLIPSE") {
                lat.reseau[i]->aperx = lat.reseau[i]->APER_1;
                lat.reseau[i]->apery = lat.reseau[i]->APER_2;
            } else if (lat.reseau[i]->APERTYPE == "RECTANGLE") {
                lat.reseau[i]->aperx = lat.reseau[i]->APER_1;
                lat.reseau[i]->apery = lat.reseau[i]->APER_2;
            } else if (lat.reseau[i]->APERTYPE == "LHCSCREEN") {
                if (lat.reseau[i]->APER_1 == 0) {
                    lat.reseau[i]->aperx = lat.reseau[i]->APER_3;
                    lat.reseau[i]->apery = lat.reseau[i]->APER_2;
                } else if (lat.reseau[i]->APER_2 == 0) {
                    lat.reseau[i]->aperx = lat.reseau[i]->APER_1;
                    lat.reseau[i]->apery = lat.reseau[i]->APER_3;
                } else {
                    cerr << "ERROR READMAD: invalid LHCSCREEN aperture definition!" << endl;
                }
            } else if (lat.reseau[i]->APERTYPE == "RECTELLIPSE") {
                if (lat.reseau[i]->APER_1 == 0) {
                    lat.reseau[i]->APER_1 = 1;
                } else if (lat.reseau[i]->APER_2 == 0) {
                    lat.reseau[i]->APER_2 = 1;
                } else if (lat.reseau[i]->APER_3 == 0) {
                    lat.reseau[i]->APER_3 = 1;
                } else if (lat.reseau[i]->APER_4 == 0) {
                    lat.reseau[i]->APER_4 = 1;
                }
                if (lat.reseau[i]->APER_1 > lat.reseau[i]->APER_3) {
                    lat.reseau[i]->aperx = lat.reseau[i]->APER_3;
                } else {
                    lat.reseau[i]->aperx = lat.reseau[i]->APER_1;
                }
                if (lat.reseau[i]->APER_2 > lat.reseau[i]->APER_4) {
                    lat.reseau[i]->apery = lat.reseau[i]->APER_4;
                } else {
                    lat.reseau[i]->apery = lat.reseau[i]->APER_2;
                }
            }
            if ((lat.reseau[i]->NAME[0] == 'T') && (lat.reseau[i]->NAME[1] == 'C') && (lat.reseau[i]->NAME[2] == 'P')) {
                lat.reseau[i]->aperx = 1;
                lat.reseau[i]->apery = 1;
            }
            if ((lat.reseau[i]->NAME[0] == 'T') && (lat.reseau[i]->NAME[1] == 'C') && (lat.reseau[i]->NAME[2] == 'S')) {
                lat.reseau[i]->aperx = 1;
                lat.reseau[i]->apery = 1;
            }
            if ((lat.reseau[i]->NAME[0] == 'T') && (lat.reseau[i]->NAME[1] == 'C') && (lat.reseau[i]->NAME[2] == 'T')) {
                lat.reseau[i]->aperx = 1;
                lat.reseau[i]->apery = 1;
            }
            if ((lat.reseau[i]->NAME[0] == 'T') && (lat.reseau[i]->NAME[1] == 'C') && (lat.reseau[i]->NAME[2] == 'L') && (lat.reseau[i]->NAME[3] == 'A')) {
                lat.reseau[i]->aperx = 1;
                lat.reseau[i]->apery = 1;
            }
            if ((lat.reseau[i]->NAME[0] == 'T') && (lat.reseau[i]->NAME[1] == 'C') && (lat.reseau[i]->NAME[2] == 'R') && (lat.reseau[i]->NAME[3] == 'Y') && (lat.reseau[i]->NAME[4] == 'O')) {
                lat.reseau[i]->aperx = 1;
                lat.reseau[i]->apery = 1;
            }
        }

        //we take into account that the elements have a length not equal to 0 to determine the real apertures, using interpolation

        vector <double> Lelem, sh1, aperx2, apery2;
        vector <int> indice;

        Lelem.push_back(0);

        for (int k(0); k < lat.reseau.size() - 1; ++k) {
            Lelem.push_back(lat.reseau[k + 1]->S - lat.reseau[k]->S);
        }

        for (int i(0); i < Lelem.size(); ++i) {
            if (Lelem[i] > 0) {

                sh1.push_back(lat.reseau[i - 1]->S + 1e10 * pow(2., -52));
                lat.aperx1.push_back(lat.reseau[i]->aperx);
                lat.apery1.push_back(lat.reseau[i]->apery);
            }
        }
        for (int j(0); j < lat.reseau.size(); ++j) {
            lat.sh.push_back(lat.reseau[j]->S);
        }
        for (int j(0); j < sh1.size(); ++j) {
            lat.sh.push_back(sh1[j]);
        }

        for (int j(0); j < lat.reseau.size(); ++j) {
            aperx2.push_back(lat.reseau[j]->aperx);
        }
        for (int j(0); j < sh1.size(); ++j) {
            aperx2.push_back(lat.aperx1[j]);
        }

        for (int j(0); j < lat.reseau.size(); ++j) {
            apery2.push_back(lat.reseau[j]->apery);
        }
        for (int j(0); j < sh1.size(); ++j) {
            apery2.push_back(lat.apery1[j]);
        }

        sh1.clear();

        for (int j(0); j < lat.sh.size(); ++j) {
            sh1.push_back(lat.sh[j]);
        }
        sort(lat.sh.begin(), lat.sh.end());

        for (int j(0); j < lat.sh.size(); ++j) {
            int m(0);
            for (int k(0); k < sh1.size(); ++k) {
                if (lat.sh[j] == sh1[k]) {
                    indice.push_back(m);
                    sh1[k] = -10;
                    break;
                } else {
                    ++m;
                }
            }
        }

        lat.aperx1.clear();
        lat.apery1.clear();

        for (int k(0); k < indice.size(); ++k) {
            for (int j(0); j < aperx2.size(); ++j) {
                if (j == indice[k]) {
                    lat.aperx1.push_back(aperx2[j]);
                    lat.apery1.push_back(apery2[j]);
                    break;
                }
            }
        }

        aperx2.clear();
        apery2.clear();


        for (int j(0); j < lat.aperx1.size(); ++j) {
            if (lat.aperx1[j] < 1) {
                aperx2.push_back(lat.aperx1[j]);
            }
        }

        double mahx=0.0;

        mahx = *max_element(aperx2.begin(), aperx2.end());

        for (int j(0); j < lat.aperx1.size(); ++j) {
            if (lat.aperx1[j] == 1) {
                lat.aperx1[j] = mahx;
            }
        }


        for (int j(0); j < lat.apery1.size(); ++j) {
            if (lat.apery1[j] < 1) {
                apery2.push_back(lat.apery1[j]);
            }
        }

        double mahy=0.0;

        mahy = *max_element(apery2.begin(), apery2.end());

        for (int j(0); j < lat.apery1.size(); ++j) {
            if (lat.apery1[j] == 1) {
                lat.apery1[j] = mahy;
            }
        }

        for (int j(0); j < lat.sh.size() - 1; ++j) {
            if (lat.sh[j + 1] - lat.sh[j] == 0) {
                lat.sh[j] = lat.sh[j] - 0.0001;
            }
        }

        for (int j(0); j < lat.reseau.size(); ++j) {
            int m(0);
            for (int k(0); k < lat.sh.size(); ++k) {
                if ((lat.reseau[j]->S < lat.sh[k]) && (lat.reseau[j]->S >= lat.sh[k - 1])) {
                    lat.reseau[j]->aperx = interp1(lat.sh[k - 1], lat.aperx1[k - 1], lat.sh[k], lat.aperx1[k], lat.reseau[j]->S);
                    lat.reseau[j]->apery = interp1(lat.sh[k - 1], lat.apery1[k - 1], lat.sh[k], lat.apery1[k], lat.reseau[j]->S);
                }
            }
        }

        indice.clear();
    } else {//if JohnJaperflag=0, we don't have apertures information, so we throw an error

        cout << "We don't have apertures informations!" << endl;

        throw 1;

    }

    vector <int> indice;

    for (int j(0); j < lat.reseau.size(); ++j) {
        if (lat.reseau[j]->KEYWORD == "HKICKER") {
            indice.push_back(j);
        }
    }

    for (int j(0); j < lat.reseau.size(); ++j) {
        if (lat.reseau[j]->KEYWORD == "KICKER") {
            indice.push_back(j);
        }
    }

    double hk=0.0;

    for (int k(0); k < indice.size(); ++k) {

        hk = lat.reseau[indice[k]]->PXC - lat.reseau[indice[k] - 1]->PXC;
        if (hk != 0) {
            lat.xcormag.push_back(hk);
            lat.ixcormag.push_back(indice[k]);
        }
    }

    indice.clear();

    for (int j(0); j < lat.reseau.size(); ++j) {
        if (lat.reseau[j]->KEYWORD == "VKICKER") {
            indice.push_back(j);
        }
    }

    for (int j(0); j < lat.reseau.size(); ++j) {
        if (lat.reseau[j]->KEYWORD == "KICKER") {
            indice.push_back(j);
        }
    }

    double vk=0.0;

    for (int k(0); k < indice.size(); ++k) {

        vk = lat.reseau[indice[k]]->PYC - lat.reseau[indice[k] - 1]->PYC;
        if (vk != 0) {
            lat.ycormag.push_back(vk);
            lat.iycormag.push_back(indice[k]);
        }
    }

}
/***************************************************************************
 * 
 * Read the lattice elements parameters
 * 
 * 
 * 
 * 
 *  Jianfeng Zhang @ LAL, 15/04/2014
 *   Fix the bug to read the optics files of the lattice element parameters.
 *   Now the code can correctly the external file until the end of the file is 
 *   arrived.
 * 
 ***************************************************************************/
void Simulation::defineParameters(string opticsFile, const int& beamflag, int& JohnJaperFlag)
{

    ifstream entree;
    string APERTYPE="", name="", KEYWORD="", PARENT="";
    entree.open(opticsFile.c_str());

    if (entree.fail()) {
        cerr << "Warning: problem with the optics file " << opticsFile << "!!" << endl;
    } else {

        string name="", temp="";
        string strtemp="", strtemp1="";
         /* read the parameters at the beginning of the MADX twiss file*/
        getline(entree, strtemp, ',');
        while ((strtemp != "NAME") && (strtemp != "ALFX") && (strtemp != "S") && (strtemp != "L") && (strtemp != "K0L") && (strtemp != "K0SL") && (strtemp != "K1L") && (strtemp != "K1SL") && (strtemp != "K2L") && (strtemp != "K2SL") && (strtemp != "XC") && (strtemp != "PXC") && (strtemp != "KEYWORD") && (strtemp != "PARENT") && (strtemp != "YC") && (strtemp != "PYC") && (strtemp != "TC") && (strtemp != "PTC") && (strtemp != "BETX") && (strtemp != "MUX") && (strtemp != "DX") && (strtemp != "DPX") && (strtemp != "BETY") && (strtemp != "ALFY") && (strtemp != "MUY") && (strtemp != "DY") && (strtemp != "DPY") && (strtemp != "APERTYPE") && (strtemp != "APER_1") && (strtemp != "APER_2") && (strtemp != "APER_3") && (strtemp != "APER_4")) {
            getline(entree, strtemp1);
            getline(entree, strtemp, ',');
        }
         /* read the parameters of the lattice elements*/
        double S=0.0, L=0.0, K0L=0.0, K0SL=0.0, K1L=0.0, K1SL=0.0, K2L=0.0, K2SL=0.0, XC=0.0, PXC=0.0, YC=0.0, PYC=0.0, TC=0.0, PTC=0.0, 
	       BETX=0.0, ALFX=0.0, MUX=0.0, DX=0.0, DPX=0.0, BETY=0.0, ALFY=0.0, MUY=0.0, DY=0.0, DPY=0.0, APER_1=0.0, APER_2=0.0, APER_3=0.0, APER_4=0.0;

        int taille(0);

        vector <string> header;
        vector <string> header_base;
        vector <string> vectemp;
        vector <int> order;

	size_t pos=0;
	std::string input;
	std::string delimiter = ",";
	
        //header_base is: NAME,KEYWORD,PARENT,S,L,K0L,K0SL,K1L,K1SL,K2L,K2SL,XC,PXC,YC,PYC,TC,PTC,BETX,ALFX,MUX,DX,DPX,BETY,ALFY,MUY,DY,DPY,APERTYPE,APER_1,APER_2,APER_3,APER_4
        //we can have any order of the headers in the opticsfile
        header_base.push_back("NAME");
        header_base.push_back("KEYWORD");
        header_base.push_back("PARENT");
        header_base.push_back("S");
        header_base.push_back("L");
        header_base.push_back("K0L");
        header_base.push_back("K0SL");
        header_base.push_back("K1L");
        header_base.push_back("K1SL");
        header_base.push_back("K2L");
        header_base.push_back("K2SL");
        header_base.push_back("XC");
        header_base.push_back("PXC");
        header_base.push_back("YC");
        header_base.push_back("PYC");
        header_base.push_back("TC");
        header_base.push_back("PTC");
        header_base.push_back("BETX");
        header_base.push_back("ALFX");
        header_base.push_back("MUX");
        header_base.push_back("DX");
        header_base.push_back("DPX");
        header_base.push_back("BETY");
        header_base.push_back("ALFY");
        header_base.push_back("MUY");
        header_base.push_back("DY");
        header_base.push_back("DPY");
        header_base.push_back("APERTYPE");
        header_base.push_back("APER_1");
        header_base.push_back("APER_2");
        header_base.push_back("APER_3");
        header_base.push_back("APER_4");

        header.push_back(strtemp);
        for (int n(0); n < 30; ++n) {
            getline(entree, strtemp, ',');
            header.push_back(strtemp);
        }
        getline(entree, strtemp);
	/* remove the "\r" symbol from the last variable in the "*.csv file", 
	       maybe this is a potential bug for other file format... By jianfeng Zhang @ LAL, 21/03/2014*/
	    strtemp = strtemp.substr(0,strtemp.size()-1);
        header.push_back(strtemp);

        for (int t(0); t < header.size(); ++t) {
            for (int q(0); q < header_base.size(); ++q) {
                if (header[t] == header_base[q]) {
                    order.push_back(q);
                    break;
                }
            }
        }

        /*start read the lattice elements parameters*/
        while (entree >> input) {
	  
               ++taille;
	    /* lattice elment*/
            while ((pos=input.find(delimiter)) !=std::string::npos) {
              strtemp = input.substr(0,pos);  
               vectemp.push_back(strtemp);
		input.erase(0,pos+delimiter.size());
            }
            /* last variable of the lattice element */
            strtemp=input;
	    vectemp.push_back(strtemp);
	   
	   
	   
            for (int h(0); h < order.size(); ++h) {

                if (order[h] == 0) {
                    name = vectemp[h];
                } else if (order[h] == 1) {
                    KEYWORD = vectemp[h];
                } else if (order[h] == 2) {
                    PARENT = vectemp[h];
                } else if (order[h] == 3) {
                    S = atof(vectemp[h].c_str());
                } else if (order[h] == 4) {
                    L = atof(vectemp[h].c_str());
                } else if (order[h] == 5) {
                    K0L = atof(vectemp[h].c_str());
                } else if (order[h] == 6) {
                    K0SL = atof(vectemp[h].c_str());
                } else if (order[h] == 7) {
                    K1L = atof(vectemp[h].c_str());
                } else if (order[h] == 8) {
                    K1SL = atof(vectemp[h].c_str());
                } else if (order[h] == 9) {
                    K2L = atof(vectemp[h].c_str());
                } else if (order[h] == 10) {
                    K2SL = atof(vectemp[h].c_str());
                } else if (order[h] == 11) {
                    XC = atof(vectemp[h].c_str());
                } else if (order[h] == 12) {
                    PXC = atof(vectemp[h].c_str());
                } else if (order[h] == 13) {
                    YC = atof(vectemp[h].c_str());
                } else if (order[h] == 14) {
                    PYC = atof(vectemp[h].c_str());
                } else if (order[h] == 15) {
                    TC = atof(vectemp[h].c_str());
                } else if (order[h] == 16) {
                    PTC = atof(vectemp[h].c_str());
                } else if (order[h] == 17) {
                    BETX = atof(vectemp[h].c_str());
                } else if (order[h] == 18) {
                    ALFX = atof(vectemp[h].c_str());
                } else if (order[h] == 19) {
                    MUX = atof(vectemp[h].c_str());
                } else if (order[h] == 20) {
                    DX = atof(vectemp[h].c_str());
                } else if (order[h] == 21) {
                    DPX = atof(vectemp[h].c_str());
                } else if (order[h] == 22) {
                    BETY = atof(vectemp[h].c_str());
                } else if (order[h] == 23) {
                    ALFY = atof(vectemp[h].c_str());
                } else if (order[h] == 24) {
                    MUY = atof(vectemp[h].c_str());
                } else if (order[h] == 25) {
                    DY = atof(vectemp[h].c_str());
                } else if (order[h] == 26) {
                    DPY = atof(vectemp[h].c_str());
                } else if (order[h] == 27) {
                    APERTYPE = vectemp[h];
                } else if (order[h] == 28) {
                    APER_1 = atof(vectemp[h].c_str());
                } else if (order[h] == 29) {
                    APER_2 = atof(vectemp[h].c_str());
                } else if (order[h] == 30) {
                    APER_3 = atof(vectemp[h].c_str());
                } else if (order[h] == 31) {
                    APER_4 = atof(vectemp[h].c_str());
                }
            }

	    
            Element elt(ALFX, ALFY, APER_1, APER_2, APER_3, APER_4, APERTYPE, BETX, BETY, DPX, DPY, DX, DY, KEYWORD, L, MUX, MUY, name, PTC, PXC, PYC, S, TC, XC, YC, K0L, K0SL, K1L, K1SL, K2L, K2SL, PARENT);

            lat.addElement(new Element(elt));

            vectemp.clear();

            

        }
    //    cout << name << "         "<<taille<<endl;
        /*number of lattice elements*/
        this ->nbre_elts = taille;
    }
    entree.close();

    if (beamflag == 2) {

        Lattice lattemp(lat);

        for (int i(0); i < nbre_elts; ++i) {

            lat.reseau[nbre_elts - 1 - i] = lattemp.reseau[i];
        }
        lattemp = lat;
        lat.reseau[0] = lattemp.reseau[nbre_elts - 1];

        for (int j(1); j < nbre_elts; ++j) {
            lat.reseau[j] = lattemp.reseau[j - 1];
        }

    }

    JohnJaperFlag = 1;//for the moment, ICOSIM++ cannot run without aperture parameters, so JohnJaperflag is fixed to 1; the variable is still here to facilitate further developpment

}



double Simulation::interp1(double x1, double y1, double x2, double y2, double x)
{

    double d1(x1 - x2);
    double d2(y1 - y2);

    double y;

    y = (x - x1) * (d2 / d1) + y1;

    return y;
}


void Simulation::genbunch(const int& i0, const int& npart, const string& partdistr, const double& r1r2skin, const double& emx, const double& emy, const double& sigdpp, const double& bx, const double& ax, const double& dx, const double& dpx, const double& by, const double& ay, const double& dy, const double& dpy, const double& nsigi, const string& inputfile)
{

  //save bunch distribution
  
  string outfile;
  outfile = inputfile + "initial.dat";
  ofstream sortie(outfile.c_str());
  
  if (sortie.fail()) {
        cerr << "Warning: problem with the output file to save the bunch distributions!!" << endl;
    }

    srand(time(0));

    Particle p1;

    for (int l(0); l < npart; ++l) {

        p1.genpartdist(i0, 1, partdistr, r1r2skin, emx, emy, sigdpp, bx, ax, dx, dpx, by, ay, dy, dpy, nsigi);

        bunch.push_back(p1);
	
	sortie <<p1.coordonnees[0][0] <<"        "<<p1.coordonnees[0][1]<<"        "<<p1.coordonnees[0][2]<<"        "<<p1.coordonnees[0][3]<<"        "<<p1.coordonnees[0][4];
	sortie << endl;
    }

    
    sortie.close();
}


void Simulation::affichebunch()
{

    for (int i(0); i < bunch.size(); ++i) {

        cout << "Particle " << i + 1 << endl;
        bunch[i].afficheFirstCoordonnees();
    }
}

/**************************************************************************************
 * 
 * 
 * 
 * 
 * 
 ***************************************************************************************/
void Simulation::writeoutput(string outputfile, const double& emix, const double& emiy, const double& Apr, const double& Zpr, const double& nparti, const double& beamflag, const double& PlossPb, const int& i0, const int& im, const int& npart, const int& kbunch, const vector <int>& asumrem, const vector <int>& asumhitcolli, const vector <int>& asumhits, const int& taubeam, vector <double>& hitso, const vector <int>& nhitcollio, vector <double>& Aphito, vector <double>& Zphito, const int& ibeg, const int& iend, vector <vector <double> >& xco, vector <vector <double> >& yco, const string& plotflag, string outputpath, string inputpath)
{

    ofstream sortie(outputfile.c_str());

    if (sortie.fail()) {
        cerr << "Warning: problem with the output file!!" << endl;
    } else {

        sortie << "Position of TCP, CRYSTAL, TCRYO collimators (ip):";

        for (int i(0); i < lat.ip.size(); ++i) {
            sortie << lat.ip[i]+1 << ", ";
        }

        sortie << endl;

        sortie << "Position of TCS collimators (is):";

        for (int i(0); i < lat.is.size(); ++i) {
            sortie << lat.is[i]+1 << ", ";
        }

        sortie << endl;

        sortie << "Position of all the collimators (ips):";

        for (int i(0); i < lat.ips.size(); ++i) {
            sortie << lat.ips[i]+1 << ", ";
        }

        sortie << endl;

        sortie << "Initial size of the beam (nsig):";

        for (int i(0); i < lat.resColli.size(); ++i) {
            sortie << lat.resColli[i]->nsig << ", ";
        }

        sortie << endl;


        sortie << "Mass number of the reference particle (Apr): " << Apr << endl;
        sortie << "Charge state of the reference particle (Zpr): " << Zpr << endl;
        sortie << "Number of particles in the simulation (nparti): " << nparti << endl;
        sortie << "Average power loss of the beam (PlossPb [J/s]): " << PlossPb << endl;

        sortie << "Mass for the particles hitting the aperture (Aphit):";

        for (int i(0); i < Aphito.size(); ++i) {
            sortie << Aphito[i] << ", ";
        }

        sortie << endl;

        sortie << "Charge of the particles hitting the aperture (Zphit):";

        for (int i(0); i < Zphito.size(); ++i) {
            sortie << Zphito[i] << ", ";
        }

        sortie << endl;

        sortie << "Number of particle staying in the accelerator for each turn (asumrem):";

        for (int i(0); i < asumrem.size(); ++i) {
            sortie << asumrem[i] << ", ";
        }

        sortie << endl;

        sortie << "Number of particles that have hit a collimator during a preceding turn (asumhitcolli):";

        for (int i(0); i < asumhitcolli.size(); ++i) {
            sortie << asumhitcolli[i] << ", ";
        }

        sortie << endl;

        sortie << "(Total) Number of particles that have hit another element during the preceding turns (asumhits):";

        for (int i(0); i < asumhits.size(); ++i) {
            sortie << asumhits[i] << ", ";
        }

        sortie << endl;

        sortie << "(Total) Number of particles that have hit the collimators, for each collimator (nhitcolli):";

        for (int i(0); i < nhitcollio.size(); ++i) {
            sortie << nhitcollio[i] << ", ";
        }

        sortie << endl;


        sortie << "Hit position along the accelerator for the particles that have hit another element than a collimator (hits):";

        for (int i(0); i < hitso.size(); ++i) {
            sortie << hitso[i] << ", ";
        }

        sortie << endl;


        sortie << "Expected lifetime of the beam (taubeam [second]): " << taubeam;

        sortie.close();

    }

    PlotRunSummary(asumrem, asumhitcolli, asumhits, nhitcollio, hitso, PlossPb, outputpath);
    PlotLossSpectra(beamflag, nparti, asumrem, Apr, Zpr, hitso, Aphito, Zphito, PlossPb, npart, kbunch, taubeam, nhitcollio, outputpath, inputpath);

    if (plotflag == "Yes") {
        PlotTrajectory(emix, emiy, ibeg, iend, xco, yco, outputpath); 
    }

}


/*************************************************************************************************
 * 
 * 
 * Comments: 
 * 
 *   (1) Fix the bugs to save ips, ip, is, it, itcla. By Jianfeng Zhang @ LAL, 16/04/2014.
 * 
 *************************************************************************************************/
void Simulation::PlotRunSummary(const vector <int>& asumrem, const vector <int>& asumhitcolli, const vector <int>& asumhits, const vector <int>& nhitcollio, vector <double>& hitso, const double& PlossPb, string outputpath)
{

    //Preparing files for the plotting of the loss map

    int NLostTotal(0);
    int irevloc(0), i0loc(0), imloc(0);

    //total number of lost particles on collimators + apertures
    NLostTotal = asumhitcolli[asumhitcolli.size() - 1] + asumhits[asumhits.size() - 1];

    irevloc = asumrem.size() - 1;
    i0loc = 0;
    imloc = lat.reseau.size() - 1;

    sort(hitso.begin(), hitso.end());

    vector <double> sep;
    vector <double> plosslocal, zlosslocal; // local lost power & lost position

    for (double k(lat.reseau[i0loc]->S); k < lat.reseau[imloc]->S; ++k) {
        sep.push_back(k);
    }

    //calculate the location range to count the lost particles in this range, each 0.5 m
    for (int i(0); i < sep.size() - 1; ++i) {
        zlosslocal.push_back(sep[i] + (sep[i + 1] - sep[i]) / 2);
    }

    int count(0);

    //find the partice lost locations and number of lost particles
    for (int i(0); i < sep.size() - 1; ++i) {
        for (int j(0); j < hitso.size(); ++j) {
            if ((hitso[j] >= sep[i]) && (hitso[j] < sep[i + 1])) {
                ++count;
            }
        }
        plosslocal.push_back(count);
        count = 0;
    }

    ofstream outplot1;
    string plot1;
    plot1 = outputpath + "/PlotRunSummary1.dat";

    outplot1.open(plot1.c_str());

    if (outplot1.fail()) {
        cerr << "Warning: problem with the file " << plot1 << "!" << endl;
    } else {

        for (int i(0); i < plosslocal.size(); ++i) {
            outplot1<< setw(10) <<  plosslocal[i]*PlossPb / NLostTotal << setw(10) << zlosslocal[i] / 1000 << endl;
        }

        outplot1.close();
    }

    vector <double> sip;
    vector <int> pos;

    for (int i(0); i < lat.reseau.size(); ++i) {
        if (lat.reseau[i]->NAME.substr(0, 2) == "IP") {
            sip.push_back(lat.reseau[i]->S);
            pos.push_back(i);
	 //   std::cout << lat.reseau[i]->NAME << std::endl;
        }
    }

  // std::cout << pos.size() << std::endl;
  // std::cout << "size of the IP number: " << sip.size() << std::endl;
    
    if (sip.size() != 0) {

        ofstream outplot2;
        string plot2;
        plot2 = outputpath + "/PlotRunSummary2.dat";

        outplot2.open(plot2.c_str());

        if (outplot2.fail()) {
            cerr << "Warning: problem with the file " << plot2 << "!" << endl;
        } else {

            for (int i(0); i < sip.size(); ++i) {
                outplot2 << setw(10) << 0.5 << setw(10) <<  sip[i] / 1000 << setw(10) << lat.reseau[pos[i]]->NAME << endl;
            }
            outplot2.close();
        }

        ofstream gnuout;
        string gnuscript;
        gnuscript = outputpath + "/labelsIP.p";

        gnuout.open(gnuscript.c_str());

        if (gnuout.fail()) {
            cerr << "Warning: problem with the file " << gnuscript << "!" << endl;
        } else {

            for (int i(0); i < sip.size(); ++i) {
                gnuout << "set label " << i + 1 << " " <<  '"' << lat.reseau[pos[i]]->NAME << '"' << """ at " << sip[i] / 1000 << ",0.6 front" << endl;
            }
            gnuout.close();
        }


        ofstream gnuoutbis;
        string gnuscriptbis;
        gnuscriptbis = outputpath + "/labelsIPend.p";

        gnuoutbis.open(gnuscriptbis.c_str());

        if (gnuoutbis.fail()) {
            cerr << "Warning: problem with the file " << gnuscriptbis << "!" << endl;
        } else {

            for (int i(0); i < sip.size(); ++i) {
                gnuoutbis << "set label " << i + 1 << " " << '"' << '"' << endl;
            }
            gnuoutbis.close();
        }
    }

    //Preparing the files for the plotting of the load on the collimators

    vector <int> removedCollimators;

    //if the open jar of the collimator is larger than 100 sigma, so it means
    // this collimator is not used in the collimation.
    for (int i(0); i < lat.resColli.size(); ++i) {
        if (lat.resColli[i]->nsig > 100) {
            removedCollimators.push_back(1);
        } else {
            removedCollimators.push_back(0);
        }
    }

    vector <int> newnhitcolli, ipsnew;

    cout << " size of nhitcollio.size():  "<<nhitcollio.size()<<endl;
    for (int i(0); i < nhitcollio.size(); ++i) {
        if (removedCollimators[i] == 0) {
            newnhitcolli.push_back(nhitcollio[i]);
            ipsnew.push_back(lat.ips[i]);
        }
    }

    //plot the particle hit the collimators with nsig <= 100.
    ofstream outplot3;
    string plot3;
    plot3 = outputpath + "/PlotRunSummary3.dat";

    outplot3.open(plot3.c_str());

    if (outplot3.fail()) {
        cerr << "Warning: problem with the file " << plot3 << "!" << endl;
    } else {

        for (int i(0); i < newnhitcolli.size(); ++i) {
	  cout << newnhitcolli[i]*PlossPb / NLostTotal << endl;
            outplot3 << setw(10) << i + 1 << setw(20) <<  newnhitcolli[i]*PlossPb / NLostTotal << setw(10) << ipsnew[i] << setw(20) << lat.reseau[ipsnew[i]]->NAME << endl;
        }
        outplot3.close();
    }

    ofstream gnuout2;
    string gnuscript2;
    gnuscript2 = outputpath + "/labelscolli.p";

    gnuout2.open(gnuscript2.c_str());

    if (gnuout2.fail()) {
        cerr << "Warning: problem with the file " << gnuscript2 << "!" << endl;
    } else {

        for (int i(0); i < newnhitcolli.size(); ++i) {
            gnuout2 << "set label " << i + 1 << " " << '"' << lat.reseau[ipsnew[i]]->NAME << '"' << """ at " << i+1 << ",13 rotate front" << endl;
        }
        gnuout2.close();
    }

    ofstream gnuout3;
    string gnuscript3;
    gnuscript3 = outputpath + "/labelscolliend.p";

    gnuout3.open(gnuscript3.c_str());

    if (gnuout3.fail()) {
        cerr << "Warning: problem with the file " << gnuscript3 << "!" << endl;
    } else {

        for (int i(0); i < newnhitcolli.size(); ++i) {
            gnuout3 << "set label " << i + 1 << " " << '"' << '"' << endl;
        }
        gnuout3.close();
    }

    //Preparing files for the plotting of the number of particles left in the beam, the number of particles that have hit a collimator and the number of particles that have hit the aperture
//asumrem[i]: particle rest in the beam
//asumhitcolli[i]: particle lost on the collimators
//asumhits[i]: particle lost elsewhere

    ofstream outplot4;
    string plot4;
    plot4 = outputpath + "/PlotRunSummary4.dat";

    outplot4.open(plot4.c_str());

    if (outplot4.fail()) {
        cerr << "Warning: problem with the file " << plot4 << "!" << endl;
    } else {

        for (int i(0); i < asumrem.size(); ++i) {
            outplot4 << setw(10) << i << setw(10) <<  asumrem[i] << setw(10) << asumhitcolli[i] << setw(10) << asumhits[i] << setw(10) << (asumrem[i]+asumhitcolli[i]+asumhits[i])<<endl;
        }
        outplot4.close();
    }

    /********************************************************
     * 
     *  Generate GNUPLOT scripts to plot the summary data.
     * ******************************************************/
    //Writing PlotRunSummary1.p

    //flag to switch to generate GUNPLOT script or not, default is false
    bool gnuplotflag = true;
    if(gnuplotflag)
    {
    ofstream PlotRun1;
    string plotrunname1;
    plotrunname1 = outputpath + "/PlotRunSummary1.p";

    PlotRun1.open(plotrunname1.c_str());

    if (PlotRun1.fail()) {
        cerr << "Warning: problem with the file " << PlotRun1 << "!" << endl;
    } else {

        PlotRun1 << "#Gnuplot script file for the PlotRunSummary (Plot of the loss map)" << endl << endl;
        PlotRun1 << "set autoscale xmax" << endl;
        PlotRun1 << "set autoscale y" << endl;
        PlotRun1 << "set key default" << endl;
        PlotRun1 << "set xrange [-1:]" << endl;
        PlotRun1 << "set ylabel \"P\' (W/m)\"" << endl;
        PlotRun1 << "set xlabel \"Distance from starting point (km)\"" << endl;
        PlotRun1 << "set title \"Loss map\"" << endl << endl;
        if (sip.size() != 0) {
            PlotRun1 << "load \'labelsIP.p\'" << endl << endl;
        }
        PlotRun1 << "plot \"PlotRunSummary1.dat\" using 2:1 title \'Losses\' with boxes";
        if (sip.size() != 0) {
            PlotRun1 << ",\"PlotRunSummary2.dat\" using 2:1 title \'IP\' with impulses" << endl << endl;
            PlotRun1 << "load \'labelsIPend.p\'" << endl;
        }
        PlotRun1.close();
    }


    //Writing PlotRunSummary2.p

    ofstream PlotRun2;
    string plotrunname2;
    plotrunname2 = outputpath + "/PlotRunSummary2.p";

    PlotRun2.open(plotrunname2.c_str());

    if (PlotRun2.fail()) {
        cerr << "Warning: problem with the file " << PlotRun2 << "!" << endl;
    } else {

        PlotRun2 << "#Gnuplot script file for the PlotRunSummary (Plot of the collimator load)" << endl << endl;
        PlotRun2 << "set autoscale" << endl;
        PlotRun2 << "set key default" << endl;
        PlotRun2 << "set style fill solid 0.5" << endl;
        PlotRun2 << "set ylabel \"P_coll (W)\"" << endl;
        PlotRun2 << "set xlabel \"Collimators\"" << endl;
        PlotRun2 << "set title \"Collimator load distribution\"" << endl << endl;
        PlotRun2 << "load \'labelscolli.p\'" << endl << endl;
        PlotRun2 << " plot \"PlotRunSummary3.dat\" using 1:2 title \'Collimator load\' with boxes" << endl << endl;
        PlotRun2 << "load \'labelscolliend.p\'" << endl;
        PlotRun2.close();
    }

    //Writing PlotRunSummary3.p

    ofstream PlotRun3;
    string plotrunname3;
    plotrunname3 = outputpath + "/PlotRunSummary3.p";

    PlotRun3.open(plotrunname3.c_str());

    if (PlotRun3.fail()) {
        cerr << "Warning: problem with the file " << PlotRun3 << "!" << endl;
    } else {

        PlotRun3 << "#Gnuplot script file for the PlotRunSummary (Plot of the  development after the first impact on collimator)" << endl << endl;
        PlotRun3 << "set autoscale" << endl;
        PlotRun3 << "set key default" << endl;
        PlotRun3 << "set ylabel \"Number of particles\"" << endl;
        PlotRun3 << "set xlabel \"N_turn\"" << endl;
        PlotRun3 << "set title \"Time Development after first impact on collimator\"" << endl << endl;
        PlotRun3 << "set logscale y" << endl << endl;
        PlotRun3 << " plot \"PlotRunSummary4.dat\" using 1:2 title \'rest particles in beam\' with line, \"PlotRunSummary4.dat\" using 1:3 title \'particles lost on collimators\' with line, \"PlotRunSummary4.dat\" using 1:4 title \'particles lost elsewhere\' with line" << endl << endl;
        PlotRun3 << "unset logscale y" << endl;
        PlotRun3.close();
    }

    }
    
}

void Simulation::PlotTrajectory(const double& emix, const double& emiy, const int& ibeg, const int& iend, vector <vector <double> >& xco, vector <vector <double> >& yco, string outputpath)
{

    vector <double> skmn, varplotx, varploty;

    for (int i(ibeg); i <= iend; ++i) {
        skmn.push_back(lat.reseau[i]->S / 1000);
        varplotx.push_back(1000 * 6 * sqrt(lat.reseau[i]->BETX * emix));
        varploty.push_back(1000 * 6 * sqrt(lat.reseau[i]->BETY * emiy));
    }

    ofstream outplot5;
    string plot5;
    plot5 = outputpath + "/PlotTrajectory1.dat";

    outplot5.open(plot5.c_str());

    if (outplot5.fail()) {
        cerr << "Warning: problem with the file " << plot5 << "!" << endl;
    } else {

        for (int i(0); i < skmn.size(); ++i) {
            outplot5 << setw(10) <<  skmn[i] << setw(10) << varplotx[i] << setw(10) << -varplotx[i] << setw(10) << varploty[i] << setw(10) << -varploty[i] << endl;
        }
        outplot5.close();
    }


    ofstream outplotxco;
    string plotxco;
    plotxco = outputpath + "/PlotTrajectory5.dat";

    outplotxco.open(plotxco.c_str());

    if (outplotxco.fail()) {
        cerr << "Warning: problem with the file " << plotxco << "!" << endl;
    } else {

        for (int i(ibeg); i < iend; ++i) {
            outplotxco << setw(16) << lat.reseau[i]->S / 1000;
            for (int j(0); j < xco[i].size(); ++j) {
                if (xco[i][j] != 100) {
                    outplotxco << setw(16) <<  1000 * xco[i][j];
                } else {
                    outplotxco << setw(16) << 0;
                }
            }
            outplotxco << endl;
        }
        outplotxco.close();
    }

    ofstream gnuout4;
    string gnuscript4;
    gnuscript4 = outputpath + "/plotxco.p";

    gnuout4.open(gnuscript4.c_str());

    if (gnuout4.fail()) {
        cerr << "Warning: problem with the file " << gnuscript4 << "!" << endl;
    } else {

        gnuout4 << "plot " <<  '"' << "PlotTrajectory1.dat" << '"' << " using 1:2 title " << '"' << "Beam envelope" << '"' << " with lines ls 1, ";
        gnuout4 << '"' << "PlotTrajectory1.dat" << '"' << " using 1:3 title " << '"' <<  '"' << " with lines ls 1,";



        for (int i(0); i < xco[0].size() - 1; ++i) {
            gnuout4 << '"' << "PlotTrajectory5.dat" << '"' << " using 1:" << i + 2 << " title " << '"' << '"' << " with lines,";
        }
        gnuout4 << '"' << "PlotTrajectory5.dat" << '"' << " using 1:" << xco[0].size() + 1 << " title " << '"' << '"' << " with lines, ";
        gnuout4.close();
    }

    ofstream outplotyco;
    string plotyco;
    plotyco = outputpath + "/PlotTrajectory6.dat";

    outplotyco.open(plotyco.c_str());

    if (outplotyco.fail()) {
        cerr << "Warning: problem with the file " << plotyco << "!" << endl;
    } else {

        for (int i(ibeg); i < iend; ++i) {
            outplotyco << setw(16) << lat.reseau[i]->S / 1000;
            for (int j(0); j < yco[i].size(); ++j) {
                if (yco[i][j] != 100) {
                    outplotyco << setw(16) <<  1000 * yco[i][j];
                } else {
                    outplotyco << setw(16) << 0;
                }
            }
            outplotyco << endl;
        }
        outplotyco.close();
    }

    ofstream gnuout5;
    string gnuscript5;
    gnuscript5 = outputpath + "/plotyco.p";

    gnuout5.open(gnuscript5.c_str());

    if (gnuout5.fail()) {
        cerr << "Warning: problem with the file " << gnuscript5 << "!" << endl;
    } else {

        gnuout5 << "plot " <<  '"' << "PlotTrajectory1.dat" << '"' << " using 1:4 title " << '"' << "Beam envelope" << '"' << " with lines ls 1, ";
        gnuout5 << '"' << "PlotTrajectory1.dat" << '"' << " using 1:5 title " << '"' <<  '"' << " with lines ls 1,";



        for (int i(0); i < yco[0].size() - 1; ++i) {
            gnuout5 << '"' << "PlotTrajectory6.dat" << '"' << " using 1:" << i + 2 << " title " << '"' << '"' << " with lines,";
        }
        gnuout5 << '"' << "PlotTrajectory6.dat" << '"' << " using 1:" << yco[0].size() + 1 << " title " << '"' << '"' << " with lines, ";
        gnuout5.close();
    }

    ofstream outplot6;
    string plot6;
    plot6 = outputpath + "/PlotTrajectory2.dat";

    outplot6.open(plot6.c_str());

    if (outplot6.fail()) {
        cerr << "Warning: problem with the file " << plot6 << "!" << endl;
    } else {

        for (int i(0); i < lat.ips.size(); ++i) {
            outplot6 << setw(15) << 15 << setw(15) << -15 << setw(15) << lat.reseau[lat.ips[i]]->S / 1000 << endl;
        }
        outplot6.close();
    }

    ofstream outplot7;
    string plot7;
    plot7 = outputpath + "/PlotTrajectory3.dat";

    outplot7.open(plot7.c_str());

    if (outplot7.fail()) {
        cerr << "Warning: problem with the file " << plot7 << "!" << endl;
    } else {

        for (int i(0); i < lat.ip.size(); ++i) {
            outplot7 << setw(15) << 15 << setw(15) << -15 << setw(15) << lat.reseau[lat.ip[i]]->S / 1000 << endl;
        }
        outplot7.close();
    }

    ofstream outplotis;
    string plotis;
    plotis = outputpath + "/PlotTrajectory8.dat";

    outplotis.open(plotis.c_str());

    if (outplotis.fail()) {
        cerr << "Warning: problem with the file " << plotis << "!" << endl;
    } else {

        for (int i(0); i < lat.is.size(); ++i) {
            outplotis << setw(15) << 15 << setw(15) << -15 << setw(15) << lat.reseau[lat.is[i]]->S / 1000 << endl;
        }
        outplotis.close();
    }

    vector <int> indice;
    vector <double> aperx1bis, apery1bis, shbis;

    for (int k(0); k < lat.sh.size(); ++k) {
        if ((lat.sh[k] > lat.reseau[ibeg]->S) && (lat.sh[k] < lat.reseau[iend]->S)) {
            indice.push_back(1);
        } else {
            indice.push_back(0);
        }
    }

    for (int k(0); k < lat.sh.size(); ++k) {
        if (indice[k] == 1) {
            aperx1bis.push_back(lat.aperx1[k]);
            apery1bis.push_back(lat.apery1[k]);
            shbis.push_back(lat.sh[k]);
        }
    }


    ofstream outplot8;
    string plot8;
    plot8 = outputpath + "/PlotTrajectory4.dat";

    outplot8.open(plot8.c_str());

    if (outplot8.fail()) {
        cerr << "Warning: problem with the file " << plot8 << "!" << endl;
    } else {

        for (int i(0); i < shbis.size(); ++i) {
            outplot8 << setw(15) << aperx1bis[i] * 1000 << setw(15) << -aperx1bis[i] * 1000 << setw(15) << apery1bis[i] * 1000 << setw(15) << -apery1bis[i] * 1000 << setw(15) << shbis[i] / 1000 << endl;
        }
        outplot8.close();
    }

    ofstream outplot9;
    string plot9;
    plot9 = outputpath + "/PlotTrajectory7.dat";

    outplot9.open(plot9.c_str());

    if (outplot9.fail()) {
        cerr << "Warning: problem with the file " << plot9 << "!" << endl;
    } else {

        for (int i(ibeg); i < iend; ++i) {
            outplot9 << setw(15) << skmn[i] << setw(15) << lat.reseau[i]->XC * 1000 << setw(15) << lat.reseau[i]->YC * 1000 << endl;
        }
        outplot9.close();
    }


    gnuout4.open(gnuscript4.c_str(), ios::out | ios::app);

    if (gnuout4.fail()) {
        cerr << "Warning: problem with the file " << gnuscript4 << "!" << endl;
    } else {
        gnuout4 << '"' << "PlotTrajectory2.dat" << '"' << " using 3:1 title " << '"' << "ips" <<  '"' << " with impulses, ";
        gnuout4 << '"' << "PlotTrajectory3.dat" << '"' << " using 3:1 title " << '"' << "ip" <<  '"' << " with impulses lt 3 lw 6,  ";
        gnuout4 << '"' << "PlotTrajectory8.dat" << '"' << " using 3:1 title " << '"' << "is" <<  '"' << " with impulses lt 4 lw 2, ";
        gnuout4 << '"' << "PlotTrajectory4.dat" << '"' << " using 5:1 title " << '"' << "Aperture" <<  '"' << " with lines ls 2,  ";
        gnuout4 << '"' << "PlotTrajectory4.dat" << '"' << " using 5:2 title " << '"' <<  '"' << " with lines ls 2, ";
        gnuout4 << '"' << "PlotTrajectory7.dat" << '"' << " using 1:2 title " << '"' << "Reference particle" <<  '"' << " with lines lt -1 lw 2" << endl;

        gnuout4.close();
    }

    gnuout5.open(gnuscript5.c_str(), ios::out | ios::app);

    if (gnuout5.fail()) {
        cerr << "Warning: problem with the file " << gnuscript5 << "!" << endl;
    } else {
        gnuout5 << '"' << "PlotTrajectory2.dat" << '"' << " using 3:1 title " << '"' << "ips" <<  '"' << " with impulses, ";
        gnuout5 << '"' << "PlotTrajectory3.dat" << '"' << " using 3:1 title " << '"' << "ip" <<  '"' << " with impulses lt 3 lw 6,  ";
        gnuout5 << '"' << "PlotTrajectory8.dat" << '"' << " using 3:1 title " << '"' << "is" <<  '"' << " with impulses lt 4 lw 2, ";
        gnuout5 << '"' << "PlotTrajectory4.dat" << '"' << " using 5:3 title " << '"' << "Aperture" <<  '"' << " with lines ls 2,  ";
        gnuout5 << '"' << "PlotTrajectory4.dat" << '"' << " using 5:4 title " << '"' <<  '"' << " with lines ls 2, ";
        gnuout5 << '"' << "PlotTrajectory7.dat" << '"' << " using 1:3 title " << '"' << "Reference particle" <<  '"' << " with lines lt -1 lw 2" << endl;

        gnuout5.close();
    }

    //Writing PlotTrajectory1.p

    ofstream PlotTraj1;
    string plottrajname1;
    plottrajname1 = outputpath + "/PlotTrajectory1.p";

    PlotTraj1.open(plottrajname1.c_str());

    if (PlotTraj1.fail()) {
        cerr << "Warning: problem with the file " << plottrajname1 << "!" << endl;
    } else {

        PlotTraj1 << "#Gnuplot script file for the PlotTrajectory in x" << endl << endl;
        PlotTraj1 << "set autoscale x" << endl;
        PlotTraj1 << "set yrange [-25:25]" << endl;
        PlotTraj1 << " set style line 1 lw 1" << endl;
        PlotTraj1 << "set ylabel \"x [mm]\"" << endl;
        PlotTraj1 << "set xlabel \"S-S_IP1 [km]\"" << endl;
        PlotTraj1 << "set title \"Trajectory in x\"" << endl << endl;
        PlotTraj1 << "set style line 2 lw 1 lt 6" << endl << endl;
        PlotTraj1 << "set key outside below" << endl << endl;
        PlotTraj1 << "load \'plotxco.p\'" << endl;
        PlotTraj1 << "unset key" << endl;
        PlotTraj1.close();
    }

    //Writing PlotTrajectory2.p

    ofstream PlotTraj2;
    string plottrajname2;
    plottrajname2 = outputpath + "/PlotTrajectory2.p";

    PlotTraj2.open(plottrajname2.c_str());

    if (PlotTraj2.fail()) {
        cerr << "Warning: problem with the file " << plottrajname2 << "!" << endl;
    } else {

        PlotTraj2 << "#Gnuplot script file for the PlotTrajectory in y" << endl << endl;
        PlotTraj2 << "set autoscale x" << endl;
        PlotTraj2 << "set yrange [-25:25]" << endl;
        PlotTraj2 << " set style line 1 lw 1" << endl;
        PlotTraj2 << "set ylabel \"y [mm]\"" << endl;
        PlotTraj2 << "set xlabel \"S-S_IP1 [km]\"" << endl;
        PlotTraj2 << "set title \"Trajectory in x\"" << endl << endl;
        PlotTraj2 << "set style line 2 lw 1 lt 6" << endl << endl;
        PlotTraj2 << "set key outside below" << endl << endl;
        PlotTraj2 << "load \'plotyco.p\'" << endl;
        PlotTraj2 << "unset key" << endl;
        PlotTraj2.close();
    }

}

/***********************************************************************************************************
 *
 * 
 * 
 * ********************************************************************************************************/
void Simulation::PlotLossSpectra(const double& beamflag, const double& nparti, const vector <int>& asumrem, const double& Apr, const double& Zpr, vector <double>& hitso, vector <double>& Aphito, vector <double>& Zphito, const double& PlossPb, const int& npart, const int& kbunch, const int& taubeam, const vector <int>& nhitcollio, string outputpath, string inputpath)
{

    vector <double> sip;//vector of s coordinates corresponding to element names starting with 'ip'
    vector <int> inip;
    vector <string> NAMES_IP;

    for (int i(0); i < lat.reseau.size(); ++i) {
        if (lat.reseau[i]->NAME.substr(0, 2) == "IP") {
            NAMES_IP.push_back(lat.reseau[i]->NAME);
            sip.push_back(lat.reseau[i]->S);
            inip.push_back(i);
        }
    }

    string interactionPoint, selflag, PowerOrPartFlag, decdPoP;
    double fragcutoff, d1, d2, s1, s2;

    ifstream entree;

    string plotname;
    plotname = inputpath + "infoPlotLossSpectra.csv";

    entree.open(plotname.c_str());

    if (entree.fail()) {
        cerr << "Warning: problem with the file 'infoPlotLossSpectra.csv'." << endl;
    } else {

        string word;
        double nbre;

        while (!entree.eof()) {

            getline(entree, word, ',');

            if (word == "interactionPoint") {
                getline(entree, interactionPoint);
            } else if (word == "d1") {
                getline(entree, word);
                d1 = atof(word.c_str());
            } else if (word == "d2") {
                getline(entree, word);
                d2 = atof(word.c_str());
            } else if (word == "selflag") {
                getline(entree, selflag);
            } else if (word == "fragcutoff") {
                getline(entree, word);
                fragcutoff = atof(word.c_str());
            } else if (word == "PowerOrPartFlag") {
                getline(entree, PowerOrPartFlag);
            } else if (word == "decdPoP") {
                getline(entree, decdPoP);
            } else if (word == "s1") {
                getline(entree, word);
                s1 = atof(word.c_str());
            } else if (word == "s2") {
                getline(entree, word);
                s2 = atof(word.c_str());
            } else {
                getline(entree, word);
            }
        }//end while(!entree.eof())
        entree.close();
    }

    double offx;
    int pos(0);

    for (int j(0); j < NAMES_IP.size(); ++j) {
        if (NAMES_IP[j] != interactionPoint) {
            ++pos;
        } else {
            break;
        }
    }


    if (inip.size() != 0) {
        offx = sip[pos];//s coordinate of the interaction point
        s1 = offx + d1;//starting point along the accelerator [m]
        s2 = offx + d2;//ending point along the accelerator [m]
    } else {
        offx = 0;
    }


    vector <double> dPoPeff;
    vector <double> hitsnew, Aphitnew, Zphitnew;

    if (decdPoP == "Bigger than reference particle") {

        for (int i(0); i < Aphito.size(); ++i) {
            dPoPeff.push_back(Zpr / Apr * Aphito[i] / Zphito[i] - 1);
            if (dPoPeff[i] < 0) {
                hitsnew.push_back(hitso[i]);
                Aphitnew.push_back(Aphito[i]);
                Zphitnew.push_back(Zphito[i]);
            }
        }
    } else if (decdPoP == "Smaller than reference particle") {
        for (int i(0); i < Aphito.size(); ++i) {
            dPoPeff.push_back(Zpr / Apr * Aphito[i] / Zphito[i] - 1);
            if (dPoPeff[i] > 0) {
                hitsnew.push_back(hitso[i]);
                Aphitnew.push_back(Aphito[i]);
                Zphitnew.push_back(Zphito[i]);
            }
        }
    }

    if (decdPoP != "Anything") {
        hitso.clear();
        Aphito.clear();
        Zphito.clear();

        for (int i(0); i < hitsnew.size(); ++i) {
            hitso.push_back(hitsnew[i]);
            Aphito.push_back(Aphitnew[i]);
            Zphito.push_back(Zphitnew[i]);
        }
    }


    int NLostTotal(0);

    for (int i(0); i < nhitcollio.size(); ++i) {
        NLostTotal += nhitcollio[i];
    }

    NLostTotal += hitso.size();

    double NLostPb;

    NLostPb = npart * kbunch / taubeam;

    vector <int> asel, zsel;
    vector <string> legarr;

    if ((Apr == 29) && (Zpr == 63)) {
        asel.push_back(63);
        asel.push_back(62);
        asel.push_back(61);
        asel.push_back(60);
        asel.push_back(59);
        asel.push_back(58);
        asel.push_back(62);
        asel.push_back(61);
        asel.push_back(60);
        asel.push_back(59);
        asel.push_back(58);
        asel.push_back(57);
        asel.push_back(61);
        asel.push_back(60);
        asel.push_back(59);
        asel.push_back(58);
        asel.push_back(57);
        asel.push_back(56);
        asel.push_back(60);
        asel.push_back(59);
        asel.push_back(58);
        asel.push_back(57);
        asel.push_back(56);
        asel.push_back(55);

        zsel.push_back(29);
        zsel.push_back(29);
        zsel.push_back(29);
        zsel.push_back(29);
        zsel.push_back(29);
        zsel.push_back(29);
        zsel.push_back(28);
        zsel.push_back(28);
        zsel.push_back(28);
        zsel.push_back(28);
        zsel.push_back(28);
        zsel.push_back(28);
        zsel.push_back(27);
        zsel.push_back(27);
        zsel.push_back(27);
        zsel.push_back(27);
        zsel.push_back(27);
        zsel.push_back(27);
        zsel.push_back(26);
        zsel.push_back(26);
        zsel.push_back(26);
        zsel.push_back(26);
        zsel.push_back(26);
        zsel.push_back(26);

        legarr.push_back("Cu^{63}");
        legarr.push_back("Cu^{62}");
        legarr.push_back("Cu^{61}");
        legarr.push_back("Cu^{60}");
        legarr.push_back("Cu^{59}");
        legarr.push_back("Cu^{58}");
        legarr.push_back("Ni^{62}");
        legarr.push_back("Ni^{61}");
        legarr.push_back("Ni^{60}");
        legarr.push_back("Ni^{59}");
        legarr.push_back("Ni^{58}");
        legarr.push_back("Ni^{57}");
        legarr.push_back("Co^{61}");
        legarr.push_back("Co^{60}");
        legarr.push_back("Co^{59}");
        legarr.push_back("Co^{58}");
        legarr.push_back("Co^{57}");
        legarr.push_back("Co^{56}");
        legarr.push_back("Fe^{60}");
        legarr.push_back("Fe^{59}");
        legarr.push_back("Fe^{58}");
        legarr.push_back("Fe^{57}");
        legarr.push_back("Fe^{56}");
        legarr.push_back("Fe^{55}");
    } else if ((Apr == 197) && (Zpr == 79)) {
        asel.push_back(197);
        asel.push_back(196);
        asel.push_back(195);
        asel.push_back(194);
        asel.push_back(193);
        asel.push_back(192);
        asel.push_back(195);
        asel.push_back(194);
        asel.push_back(193);
        asel.push_back(192);
        asel.push_back(191);
        asel.push_back(190);
        asel.push_back(193);
        asel.push_back(192);
        asel.push_back(191);
        asel.push_back(190);
        asel.push_back(189);
        asel.push_back(188);
        asel.push_back(191);
        asel.push_back(190);
        asel.push_back(189);
        asel.push_back(188);
        asel.push_back(187);
        asel.push_back(186);

        zsel.push_back(79);
        zsel.push_back(79);
        zsel.push_back(79);
        zsel.push_back(79);
        zsel.push_back(79);
        zsel.push_back(79);
        zsel.push_back(78);
        zsel.push_back(78);
        zsel.push_back(78);
        zsel.push_back(78);
        zsel.push_back(78);
        zsel.push_back(78);
        zsel.push_back(77);
        zsel.push_back(77);
        zsel.push_back(77);
        zsel.push_back(77);
        zsel.push_back(77);
        zsel.push_back(77);
        zsel.push_back(76);
        zsel.push_back(76);
        zsel.push_back(76);
        zsel.push_back(76);
        zsel.push_back(76);
        zsel.push_back(76);

        legarr.push_back("Au^{197}");
        legarr.push_back("Au^{196}");
        legarr.push_back("Au^{195}");
        legarr.push_back("Au^{194}");
        legarr.push_back("Au^{193}");
        legarr.push_back("Au^{192}");
        legarr.push_back("Pt^{195}");
        legarr.push_back("Pt^{194}");
        legarr.push_back("Pt^{193}");
        legarr.push_back("Pt^{192}");
        legarr.push_back("Pt^{191}");
        legarr.push_back("Pt^{190}");
        legarr.push_back("Ir^{193}");
        legarr.push_back("Ir^{192}");
        legarr.push_back("Ir^{191}");
        legarr.push_back("Ir^{190}");
        legarr.push_back("Ir^{189}");
        legarr.push_back("Ir^{188}");
        legarr.push_back("Os^{191}");
        legarr.push_back("Os^{190}");
        legarr.push_back("Os^{189}");
        legarr.push_back("Os^{188}");
        legarr.push_back("Os^{187}");
        legarr.push_back("Os^{186}");
    } else if ((Apr == 208) && (Zpr == 82)) {

        asel.push_back(208);
        asel.push_back(207);
        asel.push_back(206);
        asel.push_back(205);
        asel.push_back(204);
        asel.push_back(203);
        asel.push_back(202);
        asel.push_back(201);
        asel.push_back(207);
        asel.push_back(206);
        asel.push_back(205);
        asel.push_back(204);
        asel.push_back(203);
        asel.push_back(202);
        asel.push_back(201);
        asel.push_back(200);
        asel.push_back(199);
        asel.push_back(205);
        asel.push_back(204);
        asel.push_back(203);
        asel.push_back(202);
        asel.push_back(201);
        asel.push_back(200);
        asel.push_back(199);
        asel.push_back(198);
        asel.push_back(197);
        asel.push_back(202);
        asel.push_back(201);
        asel.push_back(200);
        asel.push_back(199);
        asel.push_back(198);
        asel.push_back(197);
        asel.push_back(196);
        asel.push_back(195);

        zsel.push_back(82);
        zsel.push_back(82);
        zsel.push_back(82);
        zsel.push_back(82);
        zsel.push_back(82);
        zsel.push_back(82);
        zsel.push_back(82);
        zsel.push_back(82);
        zsel.push_back(81);
        zsel.push_back(81);
        zsel.push_back(81);
        zsel.push_back(81);
        zsel.push_back(81);
        zsel.push_back(81);
        zsel.push_back(81);
        zsel.push_back(81);
        zsel.push_back(81);
        zsel.push_back(80);
        zsel.push_back(80);
        zsel.push_back(80);
        zsel.push_back(80);
        zsel.push_back(80);
        zsel.push_back(80);
        zsel.push_back(80);
        zsel.push_back(80);
        zsel.push_back(80);
        zsel.push_back(79);
        zsel.push_back(79);
        zsel.push_back(79);
        zsel.push_back(79);
        zsel.push_back(79);
        zsel.push_back(79);
        zsel.push_back(79);
        zsel.push_back(79);

        legarr.push_back("Pb^{208}");
        legarr.push_back("Pb^{207}");
        legarr.push_back("Pb^{206}");
        legarr.push_back("Pb^{205}");
        legarr.push_back("Pb^{204}");
        legarr.push_back("Pb^{203}");
        legarr.push_back("Pb^{202}");
        legarr.push_back("Pb^{201}");
        legarr.push_back("Tl^{207}");
        legarr.push_back("Tl^{206}");
        legarr.push_back("Tl^{205}");
        legarr.push_back("Tl^{204}");
        legarr.push_back("Tl^{203}");
        legarr.push_back("Tl^{202}");
        legarr.push_back("Tl^{201}");
        legarr.push_back("Tl^{200}");
        legarr.push_back("Tl^{199}");
        legarr.push_back("Hg^{205}");
        legarr.push_back("Hg^{204}");
        legarr.push_back("Hg^{203}");
        legarr.push_back("Hg^{202}");
        legarr.push_back("Hg^{201}");
        legarr.push_back("Hg^{200}");
        legarr.push_back("Hg^{199}");
        legarr.push_back("Hg^{198}");
        legarr.push_back("Hg^{197}");
        legarr.push_back("Au^{202}");
        legarr.push_back("Au^{201}");
        legarr.push_back("Au^{200}");
        legarr.push_back("Au^{199}");
        legarr.push_back("Au^{198}");
        legarr.push_back("Au^{197}");
        legarr.push_back("Au^{196}");
        legarr.push_back("Au^{195}");
    } else {

        cerr << "Warning! It is not possible to mark which isotopes are lost when the reference particle has mass number " << Apr << " and charge " << Zpr << "!" << endl;
        selflag = "No";
    }

    double aymax;
    vector <vector <double> > plosslocalt;
    vector <double> plosslocal, zlosslocal;
    vector <int> nasel, naselr;

    if (selflag == "Yes") {

        for (int i(0); i < asel.size(); ++i) {
            nasel.push_back(i);
        }

        hitsnew.clear();
        Aphitnew.clear();
        Zphitnew.clear();

        for (int i(0); i < hitso.size(); ++i) {
            if ((hitso[i] > s1) && (hitso[i] < s2)) {
                hitsnew.push_back(hitso[i]);
                Aphitnew.push_back(Aphito[i]);
                Zphitnew.push_back(Zphito[i]);
            }
        }

        vector <int> alhitsp;
        vector <double> hitsp, hitsprest, Aphitrest, Zphitrest;
        vector <int> inh;

        for (int k(0); k < nasel.size(); ++k) {
            int ia;
            ia = nasel[k];
            for (int i(0); i < hitso.size(); ++i) {
                if ((hitso[i] > s1) && (hitso[i] < s2) && (Aphito[i] == asel[ia]) && (Zphito[i] == zsel[ia])) {
                    hitsp.push_back(hitso[i]);
                }
            }
            alhitsp.push_back(hitsp.size());
            hitsp.clear();
            for (int i(0); i < Zphitnew.size(); ++i) {
                if ((Zphitnew[i] == zsel[ia]) && (Aphitnew[i] == asel[ia])) {
                    inh.push_back(1);
                } else {
                    inh.push_back(0);
                }
            }
            for (int j(0); j < hitsnew.size(); ++j) {
                if (inh[j] == 0) {
                    hitsprest.push_back(hitsnew[j]);
                    Aphitrest.push_back(Aphitnew[j]);
                    Zphitrest.push_back(Zphitnew[j]);
                }
            }
            inh.clear();
            hitsnew.clear();
            Aphitnew.clear();
            Zphitnew.clear();

            for (int j(0); j < hitsprest.size(); ++j) {
                hitsnew.push_back(hitsprest[j]);
                Aphitnew.push_back(Aphitrest[j]);
                Zphitrest.push_back(Zphitrest[j]);
            }
            hitsprest.clear();
            Aphitrest.clear();
            Zphitrest.clear();
        }
        double maxalhitsp(alhitsp[0]);

        for (int i(1); i < alhitsp.size(); ++i) {
            if (maxalhitsp < alhitsp[i]) {
                maxalhitsp = alhitsp[i];
            }
        }

        for (int i(0); i < alhitsp.size(); ++i) {
            if (alhitsp[i] > maxalhitsp * fragcutoff) {
                naselr.push_back(nasel[i]);
            }
        }

        hitsnew.clear();
        Aphitnew.clear();
        Zphitnew.clear();

        for (int i(0); i < hitso.size(); ++i) {
            if ((hitso[i] > s1) && (hitso[i] < s2)) {
                hitsnew.push_back(hitso[i]);
                Aphitnew.push_back(Aphito[i]);
                Zphitnew.push_back(Zphito[i]);
            }
        }


        alhitsp.clear();
        hitsprest.clear();
        Aphitrest.clear();
        Zphitrest.clear();
        inh.clear();

        for (int k(0); k < naselr.size(); ++k) {
            int ia;
            ia = naselr[k];
            hitsp.clear();
            for (int i(0); i < hitso.size(); ++i) {
                if ((hitso[i] > s1) && (hitso[i] < s2) && (Aphito[i] == asel[ia]) && (Zphito[i] == zsel[ia])) {
                    hitsp.push_back(hitso[i]);
                }
            }
            alhitsp.push_back(hitsp.size());

            vector <double> sep;
            zlosslocal.clear();
            for (double k(s1); k < s2; ++k) {
                sep.push_back(k);
            }

            for (int i(0); i < sep.size() - 1; ++i) {
                zlosslocal.push_back(sep[i] + (sep[i + 1] - sep[i]) / 2);
            }

            int count(0);

            for (int i(0); i < sep.size() - 1; ++i) {
                for (int j(0); j < hitsp.size(); ++j) {
                    if ((hitsp[j] >= sep[i]) && (hitsp[j] < sep[i + 1])) {
                        ++count;
                    }
                }
                plosslocal.push_back(count);
                count = 0;
            }

            plosslocalt.push_back(plosslocal);
            plosslocal.clear();

            for (int i(0); i < Zphitnew.size(); ++i) {
                if ((Zphitnew[i] == zsel[ia]) && (Aphitnew[i] == asel[ia])) {
                    inh.push_back(1);
                } else {
                    inh.push_back(0);
                }
            }
            for (int j(0); j < hitsnew.size(); ++j) {
                if (inh[j] == 0) {
                    hitsprest.push_back(hitsnew[j]);
                    Aphitrest.push_back(Aphitnew[j]);
                    Zphitrest.push_back(Zphitnew[j]);
                }
            }

            inh.clear();
            hitsnew.clear();
            Aphitnew.clear();
            Zphitnew.clear();

            for (int j(0); j < hitsprest.size(); ++j) {
                hitsnew.push_back(hitsprest[j]);
                Aphitnew.push_back(Aphitrest[j]);
                Zphitrest.push_back(Zphitrest[j]);
            }
            hitsprest.clear();
            Aphitrest.clear();
            Zphitrest.clear();

        }//end for(int k(0); k<naslr.size(); ++k)

        vector <double> sep1;
        zlosslocal.clear();
        for (double k(s1); k < s2; ++k) {
            sep1.push_back(k);
        }

        for (int i(0); i < sep1.size() - 1; ++i) {
            zlosslocal.push_back(sep1[i] + (sep1[i + 1] - sep1[i]) / 2);
        }

        int count(0);

        for (int i(0); i < sep1.size() - 1; ++i) {
            for (int j(0); j < hitsprest.size(); ++j) {
                if ((hitsprest[j] >= sep1[i]) && (hitsprest[j] < sep1[i + 1])) {
                    ++count;
                }
            }
            plosslocal.push_back(count);
            count = 0;
        }

        plosslocalt.push_back(plosslocal);
        plosslocal.clear();

        double tempsum;
        vector <double> sumploss;

        for (int l(0); l < plosslocalt.size(); ++l) {
            tempsum = plosslocalt[l][0];
            for (int h(1); h < plosslocalt[l].size(); ++h) {
                tempsum += plosslocalt[l][h];
            }
            sumploss.push_back(tempsum);
            tempsum = 0;
        }

        if (PowerOrPartFlag == "Power") {
            double aymaxtemp;
            aymax = 1.5 * sumploss[0] * PlossPb / NLostTotal;
            for (int i(1); i < sumploss.size(); ++i) {
                aymaxtemp = 1.5 * sumploss[i] * PlossPb / NLostTotal;
                if (aymaxtemp > aymax) {
                    aymax = aymaxtemp;
                }
            }
        } else if (PowerOrPartFlag == "Particles") {
            double aymaxtemp;
            aymax = (1.5 * sumploss[0] * PlossPb / NLostTotal) / 1000;
            for (int i(1); i < sumploss.size(); ++i) {
                aymaxtemp = (1.5 * sumploss[i] * PlossPb / NLostTotal) / 1000;
                if (aymaxtemp > aymax) {
                    aymax = aymaxtemp;
                }
            }
        }

    }//end if selflag == 'Yes'
    else if (selflag == "No") {

        hitsnew.clear();

        for (int i(0); i < hitso.size(); ++i) {
            if ((hitso[i] > s1) && (hitso[i] < s2)) {
                hitsnew.push_back(hitso[i]);
            }
        }

        vector <double> sep2;
        zlosslocal.clear();
        for (double k(s1); k < s2; ++k) {
            sep2.push_back(k);
        }

        for (int i(0); i < sep2.size() - 1; ++i) {
            zlosslocal.push_back(sep2[i] + (sep2[i + 1] - sep2[i]) / 2);
        }

        int count(0);

        for (int i(0); i < sep2.size() - 1; ++i) {
            for (int j(0); j < hitsnew.size(); ++j) {
                if ((hitsnew[j] >= sep2[i]) && (hitsnew[j] < sep2[i + 1])) {
                    ++count;
                }
            }
            plosslocal.push_back(count);
            count = 0;
        }

        plosslocalt.push_back(plosslocal);

        if (PowerOrPartFlag == "Power") {
            double aymaxtemp;
            aymax = 1.5 * plosslocal[0] * PlossPb / NLostTotal;
            for (int i(1); i < plosslocal.size(); ++i) {
                aymaxtemp = 1.5 * plosslocal[i] * PlossPb / NLostTotal;
                if (aymaxtemp > aymax) {
                    aymax = aymaxtemp;
                }
            }
        } else if (PowerOrPartFlag == "Particles") {
            double aymaxtemp;
            aymax = (1.5 * plosslocal[0] * PlossPb / NLostTotal) / 1000;
            for (int i(1); i < plosslocal.size(); ++i) {
                aymaxtemp = (1.5 * plosslocal[i] * PlossPb / NLostTotal) / 1000;
                if (aymaxtemp > aymax) {
                    aymax = aymaxtemp;
                }
            }
        }
    }//end if selflag == 'No'

    if (aymax <= 0) {
        aymax = 50;
    }

    double aymin(-0.4 * aymax);

    vector <vector <double> > plosslocaltransp;

    vector <double> temp;
    for (int j(0); j < plosslocalt.size(); ++j) {
        temp.push_back(0);
    }

    for (int i(0); i < plosslocalt[0].size(); ++i) {
        plosslocaltransp.push_back(temp);
    }

    temp.clear();

    for (int i(0); i < plosslocaltransp.size(); ++i) {
        for (int j(0); j < plosslocaltransp[0].size(); ++j) {
            plosslocaltransp[i][j] = plosslocalt [j][i];
        }
    }


    ofstream gnuout5;
    string gnuscript5;
    gnuscript5 = outputpath + "/PlotLossSpectra.p";

    gnuout5.open(gnuscript5.c_str());

    if (gnuout5.fail()) {
        cerr << "Warning: problem with the file " << gnuscript5 << "!" << endl;
    } else {

        gnuout5 << "#Gnuplot script file for the PlotLossSpectra(Plot of the loss map)" << endl << endl << endl;
        gnuout5 << "set autoscale y" << endl;
        gnuout5 << "set key default" << endl;
        gnuout5 << "set xrange [" << s1 << ':' << s2 << ']' << endl;
        gnuout5 << "set title " << '"' << "Loss map" << '"' << endl;
        if (PowerOrPartFlag == "Power") {
            gnuout5 << "set ylabel " << '"' << "Power load (W/m)(axis*20) / Optical function" << '"' << endl;
        } else {
            gnuout5 << "set ylabel " << '"' << "Particle loss rates (1/(ms)) / Optical function (axis /20)" << '"' << endl;
        }
        gnuout5 << "set xlabel " << '"' << "Distance along the accelerator" << '"' << endl;
        gnuout5 << "load " << "'labelsIPloss.p'" << endl;
        gnuout5 << "load " << "'labelsbendloss.p'" << endl;
        gnuout5 << "set style fill solid 0.5" << endl << endl << endl;

        gnuout5.close();
    }

    ofstream outplot10;
    string plot10;
    plot10 = outputpath + "/PlotLossSpectra1.dat";

    outplot10.open(plot10.c_str());

    if (outplot10.fail()) {
        cerr << "Warning: problem with the file " << plot10 << "!" << endl;
    } else {

        for (int i(0); i < zlosslocal.size(); ++i) {
            outplot10 << setw(15) << zlosslocal[i] - offx;
            for (int j(0); j < plosslocaltransp[0].size(); ++j) {
                if (PowerOrPartFlag == "Power") {
                    outplot10 << setw(15) << plosslocaltransp[i][j]*PlossPb / NLostTotal;
                } else {
                    outplot10 << setw(15) << plosslocaltransp[i][j]*PlossPb / NLostTotal;
                }
            }
            outplot10 << endl;
        }
        outplot10.close();
    }


    ofstream outplot11;
    string plot11;
    plot11 = outputpath + "/PlotLossSpectra2.dat";

    outplot11.open(plot11.c_str());

    if (outplot11.fail()) {
        cerr << "Warning: problem with the file " << plot11 << "!" << endl;
    } else {

        for (int i(0); i < lat.reseau.size(); ++i) {
            if ((lat.reseau[i]->S > s1) && (lat.reseau[i]->S < s2)) {
                outplot11 << setw(15) << lat.reseau[i]->S - offx << setw(15) << lat.reseau[i]->aperx * 50 << setw(15) << abs(lat.reseau[i]->DX) << setw(15) << lat.reseau[i]->BETX / 120 << endl;
            }
        }
        outplot11.close();
    }

    ofstream outplot12;
    string plot12;
    plot12 = outputpath + "/PlotLossSpectra3.dat";

    outplot12.open(plot12.c_str());

    if (outplot12.fail()) {
        cerr << "Warning: problem with the file " << plot12 << "!" << endl;
    } else {

        for (int i(0); i < sip.size(); ++i) {
            outplot12 << setw(10) << 0.5 << setw(10) <<  sip[i] - offx << endl;
        }
        outplot12.close();
    }

    ofstream gnuout7;
    string gnuscript7;
    gnuscript7 = outputpath + "/labelsIPloss.p";

    gnuout7.open(gnuscript7.c_str());

    if (gnuout7.fail()) {
        cerr << "Warning: problem with the file " << gnuscript7 << "!" << endl;
    } else {

        for (int i(0); i < sip.size(); ++i) {
            gnuout7 << "set label " << i + 1 << " " <<  '"' << NAMES_IP[i] << '"' << """ at " << sip[i] << ",0.6 front" << endl;
        }
        gnuout7.close();
    }

    ofstream gnuout6;
    string gnuscript6;
    gnuscript6 = outputpath + "/PlotLossSpectra.p";

    gnuout6.open(gnuscript6.c_str(), ios::out | ios::app);

    if (gnuout6.fail()) {
        cerr << "Warning: problem with the file " << gnuscript6 << "!" << endl;
    } else {
        if (selflag == "Yes") {

            if (naselr. size() != 0) {
                gnuout6 << "plot " <<  '"' << "PlotLossSpectra1.dat" << '"' << " using 1:2 title " << '"' << legarr[naselr[0]] << '"' << " with boxes, ";

                for (int i(1); i < naselr.size(); ++i) {
                    gnuout6 << '"' << "PlotLossSpectra1.dat" << '"' << " using 1:" << i + 2 << " title " << '"' << legarr[naselr[i]] << '"' << " with boxes, ";
                }
                gnuout6 << '"' << "PlotLossSpectra1.dat" << '"' << " using 1:" << plosslocalt.size() + 1 << " title " << '"' << "others" << '"' << " with boxes, ";
            } else {
                gnuout6 << "plot" << '"' << "PlotLossSpectra1.dat" << '"' << " using 1:" << plosslocalt.size() + 1 << " title " << '"' << "others" << '"' << " with boxes, ";
            }
        } else {
            gnuout6 << "plot " <<  '"' << "PlotLossSpectra1.dat" << '"' << " using 1:2" << " title " << '"' << "others" << '"' << " with boxes, ";
        }

        gnuout6 << '"' << "PlotLossSpectra2.dat" << '"' << " using 1:2" << " title " << '"' << "Aperture in x direction [m/50]" << '"' << " with lines, ";
        gnuout6 << '"' << "PlotLossSpectra2.dat" << '"' << " using 1:3" << " title " << '"' << "Dispersion function in x direction [m]" << '"' << " with lines, ";
        gnuout6 << '"' << "PlotLossSpectra2.dat" << '"' << " using 1:4" << " title " << '"' << "Beta function in x direction [120m]" << '"' << " with lines, ";
        if (sip.size() != 0) {
            gnuout6 << '"' << "PlotLossSpectra3.dat" << '"' << " using 2:1" << " title " << '"' << "IP" << '"' << " with impulses,  ";
        }

        gnuout6.close();
    }

    vector <double> posx, precposx;
    vector <string> names;

    if (s2 - s1 < 2000) {

        vector <int> which;

        for (int i(0); i < lat.reseau.size(); ++i) {
            if ((lat.reseau[i]->S > s1) && (lat.reseau[i]->S < s2)) {
                which.push_back(1);
            } else {
                which.push_back(0);
            }
        }



        for (int i(0); i < which.size(); ++i) {
            if (which[i] == 1) {
                if ((lat.reseau[i]->KEYWORD == "RBEND") || (lat.reseau[i]->KEYWORD == "SBEND") || (lat.reseau[i]->KEYWORD == "QUADRUPOLE") || (lat.reseau[i]->KEYWORD == "RCOLLIMATOR")) {

                    posx.push_back(lat.reseau[i]->S);
                    precposx.push_back(lat.reseau[i - 1]->S);
                    names.push_back(lat.reseau[i]->NAME);

                }
            }
        }



        ofstream outplot14;
        string plot14;
        plot14 = outputpath + "/PlotLossSpectra4.dat";

        outplot14.open(plot14.c_str());

        if (outplot14.fail()) {
            cerr << "Warning: problem with the file " << plot14 << "!" << endl;
        } else {

            for (int i(0); i < posx.size(); ++i) {
                outplot14 << setw(15) <<  precposx[i] + (posx[i] - precposx[i]) / 2 << setw(15) << (posx[i] - precposx[i]) / 2 << setw(10) << -0.2 << setw(10) << 0.1  << endl;
            }
            outplot14.close();
        }

    }

    ofstream gnuout9;
    string gnuscript9;
    gnuscript9 = outputpath + "/labelsbendloss.p";

    gnuout9.open(gnuscript9.c_str());

    if (gnuout9.fail()) {
        cerr << "Warning: problem with the file " << gnuscript9 << "!" << endl;
    } else {

        for (int i(0); i < names.size(); ++i) {
            gnuout9 << "set label " << i + 1 + sip.size() + 2 << " " <<  '"' << names[i] << '"' << """ at " << precposx[i] + (posx[i] - precposx[i]) / 2 << ",-0.3 rotate front " << endl;
        }
        gnuout9.close();
    }

    ofstream gnuout8;
    string gnuscript8;
    gnuscript8 = outputpath + "/PlotLossSpectra.p";

    gnuout8.open(gnuscript8.c_str(), ios::out | ios::app);

    if (gnuout8.fail()) {
        cerr << "Warning: problem with the file " << gnuscript8 << "!" << endl;
    } else {
        gnuout8 << '"' << "PlotLossSpectra4.dat" << '"' << " using 1:3:2:4" << " title " << '"' << '"' << " with boxxyerrorbars " << endl;


        gnuout8.close();
    }

}
